package com.jrl.fuelprice.data.gasstations

import android.util.Log
import com.jrl.fuelprice.data.alerts.local.AlertLocalSource
import com.jrl.fuelprice.data.alerts.local.entities.AlertDbModel
import com.jrl.fuelprice.data.discounts.db.DiscountsDbSource
import com.jrl.fuelprice.data.discounts.db.entities.DiscountsWithBrandDbModel
import com.jrl.fuelprice.data.gasstations.db.GasStationDbSource
import com.jrl.fuelprice.data.gasstations.db.entities.BrandDbModel
import com.jrl.fuelprice.data.gasstations.db.entities.FuelTypeDbEnum
import com.jrl.fuelprice.data.gasstations.local.GasStationLocalSource
import com.jrl.fuelprice.data.gasstations.remote.GasStationRemoteSource
import com.jrl.fuelprice.data.rating.db.RatingDbSource
import com.jrl.fuelprice.data.rating.firestore.RatingRemoteSource
import com.jrl.fuelprice.data.rating.firestore.models.RatingFirestoreModel.Companion.toDb
import com.jrl.fuelprice.data.rating.local.RatingLocalSource
import com.jrl.fuelprice.domain.gasstations.GasStationRepository
import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.GasStationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.RadioDomainEnum
import com.jrl.fuelprice.domain.rating.RatingTypeDomainEnum
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.map
import java.time.LocalDateTime
import javax.inject.Inject

class GasStationRepositoryImpl @Inject constructor(
    private val gasStationLocalSource: GasStationLocalSource,
    private val gasStationDbSource: GasStationDbSource,
    private val gasStationRemoteSource: GasStationRemoteSource,
    private val ratingLocalSource: RatingLocalSource,
    private val ratingDbSource: RatingDbSource,
    private val ratingRemoteSource: RatingRemoteSource,
    private val alertLocalSource: AlertLocalSource,
    private val discountsDbSource: DiscountsDbSource,
) : GasStationRepository {
    
    private var activeDiscounts: List<DiscountsWithBrandDbModel> = emptyList()
    
    override suspend fun getGasStationPagination(
        fuelType: FuelTypeDomainEnum,
        radio: RadioDomainEnum,
        location: LocationDomainModel?,
        fromIndex: Int,
    ): Flow<List<GasStationDomainModel>> {
        loadActiveDiscounts()
        loadGasStationsAndRatings()
        return flow {
            gasStationDbSource.getPagingGasStations(
                fuelType = fuelType,
                radio = radio,
                location = location,
                fromIndex = fromIndex,
            ).map {
                it.mapNotNull {
                    it.toDomain(
                        currentLocation = location,
                        activeDiscounts = activeDiscounts,
                    )
                }.sortedBy { it.prices.dieselA }
            }.collect(::emit)
        }
    }
    
    override suspend fun getGasStationById(
        id: Int,
        fuelType: FuelTypeDomainEnum,
        location: LocationDomainModel?,
    ): Result<GasStationDomainModel> {
        loadActiveDiscounts()
        return gasStationDbSource.getGasStationById(
            id = id,
            fuelType = fuelType,
            location = location,
        ).toDomain(
            currentLocation = location,
            activeDiscounts = activeDiscounts,
        )?.let {
            Result.success(it)
        } ?: Result.failure(Exception("Unknown error mapping from db to domain"))
    }
    
    override suspend fun getFavoritesGasStations(
        fuelType: FuelTypeDomainEnum,
        location: LocationDomainModel?,
    ): Flow<List<GasStationDomainModel>> = flow {
        gasStationDbSource.getFavorites(
            fuelType = fuelType,
            location = location,
        ).map {
            it.mapNotNull {
                it.toDomain(
                    currentLocation = location,
                    activeDiscounts = activeDiscounts,
                )
            }.sortedBy { it.prices.dieselA }
        }
    }
    
    override suspend fun saveFavoriteGasStation(
        gasStationId: Int,
        gasStationName: String,
        fuelType: FuelTypeDomainEnum,
        isFavorite: Boolean,
    ) {
        gasStationDbSource.updateFavoriteGasStation(gasStationId, isFavorite)
        alertLocalSource.saveAlert(
            AlertDbModel(
                gasStationId = gasStationId,
                gasStationName = gasStationName,
                fuelType = FuelTypeDbEnum.toDb(fuelType),
                isEnabled = false,
            )
        )
    }
    
    override suspend fun saveRatingGasStation(gasStationId: Int, rating: Int, userId: String) {
        ratingRemoteSource.saveRating(
            itemId = gasStationId,
            rating = rating,
            type = RatingTypeDomainEnum.GAS_STATION,
            userId = userId,
        )
        ratingRemoteSource.getRatings()
    }
    
    private fun loadActiveDiscounts() {
        activeDiscounts = discountsDbSource.getActiveDiscounts()
    }
    
    private suspend fun loadGasStationsAndRatings() {
        loadRatings()
        loadGasStations()
    }
    
    private suspend fun loadGasStations(): Result<Unit> {
        return gasStationLocalSource.getLastUpdate().let { lastUpdate ->
            if (lastUpdate == null || lastUpdate.plusHours(6).isBefore(LocalDateTime.now())) {
                gasStationRemoteSource.getStations().fold(
                    onSuccess = { gasStations ->
                        val brands = gasStationDbSource.getBrands()
                        val gasStationsMapped = gasStations.mapNotNull {
                            it.toDb(brandId = getBrandId(brands, it.brandName))
                        }
                        gasStationDbSource.saveAll(gasStationsMapped)
                        gasStationLocalSource.saveLastUpdate(LocalDateTime.now())
                        Result.success(Unit)
                    },
                    onFailure = {
                        Log.e("GasStationRepositoryImpl", "Error loading gas stations", it)
                        Result.failure(it)
                    }
                )
            } else {
                Result.success(Unit)
            }
        }
    }
    
    private suspend fun loadRatings(): Result<Unit> {
        return ratingRemoteSource.getRatings()
            .fold(
                onSuccess = { ratings ->
                    ratingDbSource.saveRatings(ratings.mapNotNull { it.toDb() })
                    ratingLocalSource.saveLastUpdate(LocalDateTime.now())
                    Result.success(Unit)
                },
                onFailure = {
                    Log.e("GasStationRepositoryImpl", "Error loading rating gas stations", it)
                    Result.failure(it)
                }
            )
    }
    
    private fun getBrandId(brands: List<BrandDbModel>, brandName: String?): Int {
        return brands.firstOrNull { it.name.equals(brandName, ignoreCase = true) }?.id
            ?: brands.firstOrNull { it.name == "OTRAS" }!!.id
    }
}