package com.jrl.fuelprice.data.gasstations.db.entities

import androidx.room.ColumnInfo
import androidx.room.Embedded
import androidx.room.Entity
import androidx.room.PrimaryKey
import com.jrl.fuelprice.data.discounts.db.entities.DiscountsWithBrandDbModel
import com.jrl.fuelprice.data.gasstations.db.entities.PlaceDbModel.Companion.toDb
import com.jrl.fuelprice.data.gasstations.db.entities.PriceDbModel.Companion.toDb
import com.jrl.fuelprice.data.gasstations.db.entities.SaleTypeDbEnum.Companion.toDb
import com.jrl.fuelprice.data.gasstations.db.entities.SaleTypeDbEnum.Companion.toDomain
import com.jrl.fuelprice.data.rating.db.entities.RatingDbModel
import com.jrl.fuelprice.domain.alerts.models.AlertGasStationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.BrandDomainModel
import com.jrl.fuelprice.domain.gasstations.models.GasStationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import java.time.LocalDateTime

@Entity(tableName = "gas_stations")
data class GasStationDbModel(
    @PrimaryKey @ColumnInfo(name = "id") val id: Int,
    @ColumnInfo(name = "name") val name: String,
    @Embedded val place: PlaceDbModel,
    @ColumnInfo(name = "schedule") val schedule: String? = null,
    @Embedded val prices: PriceDbModel,
    @ColumnInfo(name = "sale_type") val saleType: SaleTypeDbEnum,
    @ColumnInfo(name = "brand_id") val brandId: Int,
    @ColumnInfo(name = "is_favorite") val isFavorite: Boolean,
    @ColumnInfo(name = "update_at") val updateAt: LocalDateTime,
) {
    
    companion object {
        fun toDb(item: GasStationDomainModel): GasStationDbModel {
            return GasStationDbModel(
                id = item.id,
                name = item.name,
                place = item.place.toDb(),
                schedule = item.schedule,
                prices = item.prices.toDb(),
                saleType = item.saleType.toDb(),
                brandId = item.brand.id,
                isFavorite = item.isFavorite,
                updateAt = item.updateAt,
            )
        }
        
        fun GasStationDbModel.toDomain(
            brand: BrandDomainModel,
            ratings: List<RatingDbModel>,
            currentLocation: LocationDomainModel?,
            activeDiscounts: List<DiscountsWithBrandDbModel>,
        ): GasStationDomainModel? {
            val placeMapped = place.toDomain(currentLocation)
            val pricesMapped =
                prices.toDomain(activeDiscount = activeDiscounts.firstOrNull { it.brand.id == brandId }?.discount)
            if (placeMapped == null || pricesMapped == null) {
                return null
            }
            return try {
                GasStationDomainModel(
                    id = this.id,
                    name = this.name,
                    place = placeMapped,
                    schedule = schedule,
                    prices = pricesMapped,
                    saleType = this.saleType.toDomain(),
                    brand = brand,
                    isFavorite = this.isFavorite,
                    score = ratings.sumOf { it.score.toDouble() } / ratings.size,
                    updateAt = this.updateAt,
                )
            } catch (e: Exception) {
                null
            }
        }
        
        fun GasStationDbModel.toAlertDomain(
            brand: BrandDomainModel,
            currentLocation: LocationDomainModel?,
            isEnabled: Boolean,
        ): AlertGasStationDomainModel? {
            val placeMapped = place.toDomain(currentLocation)
            val pricesMapped = prices.toDomain()
            if (placeMapped == null || pricesMapped == null) {
                return null
            }
            return try {
                AlertGasStationDomainModel(
                    gasStation = GasStationDomainModel(
                        id = id,
                        name = name,
                        place = placeMapped,
                        schedule = schedule,
                        prices = pricesMapped,
                        saleType = saleType.toDomain(),
                        brand = brand,
                        isFavorite = false,
                        score = 0.0,
                        updateAt = LocalDateTime.now(),
                    ),
                    isEnabled = isEnabled,
                )
            } catch (e: Exception) {
                null
            }
        }
    }
}