package com.jrl.fuelprice.data.gasstations.remote.models

import com.google.gson.annotations.SerializedName
import com.jrl.fuelprice.data.gasstations.db.entities.GasStationDbModel
import com.jrl.fuelprice.data.gasstations.db.entities.PlaceDbModel
import com.jrl.fuelprice.data.gasstations.db.entities.PriceDbModel
import com.jrl.fuelprice.domain.gasstations.models.BrandDomainModel
import com.jrl.fuelprice.domain.gasstations.models.GasStationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.GasStationPlaceDomainModel
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.PriceDomainModel
import java.lang.Math.toRadians
import java.time.LocalDateTime
import kotlin.math.cos
import kotlin.math.sin

data class GasStationResponse(
    @SerializedName("C.P.") val postalCode: String? = null,
    @SerializedName("Dirección") val address: String? = null,
    @SerializedName("Horario") val schedule: String? = null,
    @SerializedName("Latitud") val latitude: String? = null,
    @SerializedName("Localidad") val locality: String? = null,
    @SerializedName("Longitud (WGS84)") val longitude: String? = null,
    @SerializedName("Margen") val margin: String? = null,
    @SerializedName("Municipio") val municipalityName: String? = null,
    @SerializedName("Precio Biodiesel") val biodieselPrice: String? = null,
    @SerializedName("Precio Bioetanol") val bioethanolPrice: String? = null,
    @SerializedName("Precio Gas Natural Comprimido") val compressedNaturalGasPrice: String? = null,
    @SerializedName("Precio Gas Natural Licuado") val liquefiedNaturalGasPrice: String? = null,
    @SerializedName("Precio Gases licuados del petróleo") val liquefiedPetroleumGasesPrice: String? = null,
    @SerializedName("Precio Gasoleo A") val dieselAPrice: String? = null,
    @SerializedName("Precio Gasoleo B") val dieselBPrice: String? = null,
    @SerializedName("Precio Gasoleo Premium") val dieselPremiumPrice: String? = null,
    @SerializedName("Precio Gasolina 95 E10") val gasoline95E10Price: String? = null,
    @SerializedName("Precio Gasolina 95 E5") val gasoline95E5Price: String? = null,
    @SerializedName("Precio Gasolina 95 E5 Premium") val gasoline95E5PremiumPrice: String? = null,
    @SerializedName("Precio Gasolina 98 E10") val gasoline98E10Price: String? = null,
    @SerializedName("Precio Gasolina 98 E5") val gasoline98E5Price: String? = null,
    @SerializedName("Precio Hidrogeno") val hydrogenPrice: String? = null,
    @SerializedName("Provincia") val province: String? = null,
    @SerializedName("Remisión") val referral: String? = null,
    @SerializedName("Rótulo") val brandName: String? = null,
    @SerializedName("Tipo Venta") val saleType: String? = null,
    @SerializedName("% BioEtanol") val bioethanolPercentage: String? = null,
    @SerializedName("% Éster metílico") val methylEsterPercentage: String? = null,
    @SerializedName("IDEESS") val gasStationId: Int? = null,
    @SerializedName("IDMunicipio") val municipalityId: String? = null,
    @SerializedName("IDProvincia") val provinceId: String? = null,
    @SerializedName("IDCCAA") val autonomousCommunityId: String? = null,
) {
    fun toDomain(): GasStationDomainModel? =
        try {
            GasStationDomainModel(
                id = requireNotNull(gasStationId),
                name = requireNotNull(brandName),
                place = GasStationPlaceDomainModel(
                    municipalityId = requireNotNull(municipalityId?.toIntOrNull()),
                    municipalityName = requireNotNull(municipalityName),
                    postalCode = requireNotNull(postalCode),
                    address = requireNotNull(address),
                    location = LocationDomainModel(
                        latitude = requireNotNull(latitude?.replace(",", ".")?.toDoubleOrNull()),
                        longitude = requireNotNull(longitude?.replace(",", ".")?.toDoubleOrNull()),
                    ),
                    distanceKm = 0.0,
                ),
                schedule = schedule,
                prices = PriceDomainModel(
                    biodiesel = biodieselPrice?.replace(",", ".")?.toDoubleOrNull(),
                    dieselA = dieselAPrice?.replace(",", ".")?.toDoubleOrNull(),
                    dieselB = dieselBPrice?.replace(",", ".")?.toDoubleOrNull(),
                    dieselPremium = dieselPremiumPrice?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline95e5 = gasoline95E5Price?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline95e5Premium = gasoline95E5PremiumPrice?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline95e10 = gasoline95E10Price?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline98e5 = gasoline98E5Price?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline98e10 = gasoline98E10Price?.replace(",", ".")?.toDoubleOrNull(),
                    hydrogen = hydrogenPrice?.replace(",", ".")?.toDoubleOrNull(),
                ),
                saleType = SaleTypeResponse.toDomain(requireNotNull(saleType)) ?: error("saleType is null"),
                brand = BrandDomainModel(
                    id = 0,
                    name = brandName
                ),
                isFavorite = false,
                score = 0.0,
                updateAt = LocalDateTime.now(),
            )
        } catch (e: Exception) {
            null
        }
    
    fun toDb(brandId: Int): GasStationDbModel? =
        try {
            val latitude = requireNotNull(latitude?.replace(",", ".")?.toDoubleOrNull())
            val longitude = requireNotNull(longitude?.replace(",", ".")?.toDoubleOrNull())
            GasStationDbModel(
                id = requireNotNull(gasStationId),
                name = requireNotNull(brandName),
                place = PlaceDbModel(
                    municipalityId = requireNotNull(municipalityId?.toIntOrNull()),
                    municipalityName = requireNotNull(municipalityName),
                    postalCode = requireNotNull(postalCode),
                    address = requireNotNull(address),
                    latitude = latitude,
                    latitudeSinRad = sin(toRadians(latitude)),
                    latitudeCosRad = cos(toRadians(latitude)),
                    longitude = longitude,
                    longitudeSinRad = sin(toRadians(longitude)),
                    longitudeCosRad = cos(toRadians(longitude)),
                ),
                schedule = schedule,
                prices = PriceDbModel(
                    biodiesel = biodieselPrice?.replace(",", ".")?.toDoubleOrNull(),
                    dieselA = dieselAPrice?.replace(",", ".")?.toDoubleOrNull(),
                    dieselB = dieselBPrice?.replace(",", ".")?.toDoubleOrNull(),
                    dieselPremium = dieselPremiumPrice?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline95e5 = gasoline95E5Price?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline95e5Premium = gasoline95E5PremiumPrice?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline95e10 = gasoline95E10Price?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline98e5 = gasoline98E5Price?.replace(",", ".")?.toDoubleOrNull(),
                    gasoline98e10 = gasoline98E10Price?.replace(",", ".")?.toDoubleOrNull(),
                    hydrogen = hydrogenPrice?.replace(",", ".")?.toDoubleOrNull(),
                ),
                saleType = SaleTypeResponse.toDb(requireNotNull(saleType)) ?: error("saleType is null"),
                brandId = brandId,
                isFavorite = false,
                updateAt = LocalDateTime.now(),
            )
        } catch (e: Exception) {
            null
        }
}