package com.jrl.fuelprice.presentation.alerts

import android.Manifest
import android.annotation.SuppressLint
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.ExperimentalLayoutApi
import androidx.compose.foundation.layout.FlowRow
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableDoubleStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.drawBehind
import androidx.compose.ui.geometry.CornerRadius
import androidx.compose.ui.graphics.PathEffect
import androidx.compose.ui.graphics.drawscope.Stroke
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import com.google.accompanist.permissions.ExperimentalPermissionsApi
import com.google.accompanist.permissions.rememberMultiplePermissionsState
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import com.google.common.collect.ImmutableList
import com.jrl.fuelprice.R
import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.presentation.alerts.models.AlertGasStationUiModel
import com.jrl.fuelprice.presentation.commons.composables.ErrorComposable
import com.jrl.fuelprice.presentation.commons.composables.ItemGasStationComposable
import com.jrl.fuelprice.presentation.commons.composables.LoadingComposable
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.permissions.location.LocationState
import com.jrl.fuelprice.presentation.commons.permissions.notifications.RequestNotificationsPermission
import com.jrl.fuelprice.presentation.commons.theme.CustomColor
import com.jrl.fuelprice.presentation.commons.theme.CustomTextStyle
import com.jrl.fuelprice.presentation.commons.theme.Dimension
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.gasstationdetails.models.GasStationUiModel
import com.jrl.fuelprice.presentation.listprices.composables.FuelTypeDropdownMenu
import com.jrl.fuelprice.presentation.navigationmaps.models.NavigationMapsUiModel
import kotlinx.coroutines.launch

@Composable
fun AlertScreen(
    onClickDetails: (gasStationId: Int) -> Unit,
    onClickNavigateMaps: (NavigationMapsUiModel) -> Unit,
    modifier: Modifier = Modifier,
    viewModel: AlertViewModel = hiltViewModel()
) {
    Surface(
        modifier = modifier,
        color = CustomColor.Background
    ) {
        var locationPermissionGranted by remember { mutableStateOf(false) }
        val fusedLocationClient = LocationServices.getFusedLocationProviderClient(LocalContext.current)
        var locationState by remember { mutableStateOf<LocationState?>(null) }
        var notificationPermissionGranted by remember { mutableStateOf(false) }
        
        LaunchedEffect(Unit) {
            viewModel.submitAction(AlertUiAction.Load)
        }
        RequestLocationPermission(
            onPermissionsGranted = { locationPermissionGranted = true },
            onPermissionsNotGranted = { locationPermissionGranted = false },
        )
        RequestNotificationsPermission(
            onPermissionsGranted = { notificationPermissionGranted = true },
            onPermissionsNotGranted = { notificationPermissionGranted = false }
        )
        viewModel.uiStateFlow.collectAsState().value.let { state ->
            when (state) {
                UiState.Loading -> LoadingComposable()
                is UiState.Success -> {
                    AlertsSuccessComposable(
                        uiModel = state.data,
                        onChangePriceToAlert = {
                            viewModel.submitAction(AlertUiAction.OnChangePriceToAlert(it))
                        },
                        onClickAlert = {
                            if (notificationPermissionGranted) {
                                viewModel.submitAction(AlertUiAction.OnClickAlert(it))
                            }
                        },
                        onClickFuelType = {
                            viewModel.submitAction(AlertUiAction.OnClickFuelType(it))
                        },
                        onClickFavorite = {
                            viewModel.submitAction(AlertUiAction.OnClickFavorite(it))
                        },
                        onClickDetails = onClickDetails,
                        onClickNavigateMaps = onClickNavigateMaps,
                    )
                }
                is UiState.Error -> ErrorComposable()
            }
            
        }
        if (locationPermissionGranted) {
            GetLocation(
                fusedLocationClient = fusedLocationClient,
                onLocation = {
                    locationState = it
                    viewModel.submitAction(AlertUiAction.OnLocation(it))
                }
            )
        }
    }
}

@OptIn(ExperimentalPermissionsApi::class)
@Composable
fun RequestLocationPermission(
    onPermissionsGranted: () -> Unit,
    onPermissionsNotGranted: () -> Unit = {}
) {
    val permissionsState = rememberMultiplePermissionsState(
        permissions = listOf(
            Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_FINE_LOCATION
        )
    )
    
    val coroutineScope = rememberCoroutineScope()
    
    LaunchedEffect(key1 = permissionsState.allPermissionsGranted) {
        if (permissionsState.allPermissionsGranted) {
            onPermissionsGranted()
        } else {
            coroutineScope.launch {
                permissionsState.launchMultiplePermissionRequest()
                onPermissionsNotGranted()
            }
        }
    }
}

@SuppressLint("MissingPermission")
@Composable
fun GetLocation(
    fusedLocationClient: FusedLocationProviderClient,
    onLocation: (location: LocationState) -> Unit = {},
) {
    var location by remember { mutableStateOf<LocationState>(LocationState.Loading) }
    
    LaunchedEffect(key1 = true) {
        fusedLocationClient.lastLocation
            .addOnSuccessListener { loc ->
                location = if (loc != null) {
                    LocationState.Success(loc)
                } else {
                    LocationState.Error("Location not found.")
                }
                onLocation(location)
            }
            .addOnFailureListener { exception ->
                location = LocationState.Error(exception.message ?: "Unknown error.")
                onLocation(location)
            }
    }
}

@OptIn(ExperimentalFoundationApi::class, ExperimentalLayoutApi::class)
@Composable
fun AlertsSuccessComposable(
    uiModel: AlertUiModel,
    onChangePriceToAlert: (priceToAlert: Double) -> Unit,
    onClickAlert: (item: AlertGasStationUiModel) -> Unit,
    onClickFuelType: (fuelType: FuelTypeDomainEnum) -> Unit,
    onClickFavorite: (item: GasStationUiModel) -> Unit,
    onClickDetails: (gasStationId: Int) -> Unit,
    onClickNavigateMaps: (NavigationMapsUiModel) -> Unit,
) {
    val scrollState = rememberLazyListState()
    Column(
        modifier = Modifier
            .fillMaxSize()
            .padding(Dimension.Padding.Normal),
    ) {
        FlowRow(
            modifier = Modifier.fillMaxWidth(),
            horizontalArrangement = Arrangement.spacedBy(space = Dimension.Padding.Little),
        ) {
            FuelTypeDropdownMenu(
                itemSelected = uiModel.fuelTypeSelected,
                items = uiModel.fuelTypesToShow,
                onClick = onClickFuelType,
            )
        }
        LazyColumn(
            modifier = Modifier.fillMaxSize(),
            state = scrollState,
        ) {
            item {
                var rememberPriceToAlert by remember { mutableDoubleStateOf(uiModel.priceToAlert) }
                Row(
                    modifier = Modifier
                        .padding(top = 1.dp, bottom = Dimension.Padding.Big)
                        .fillMaxWidth()
                        .background(CustomColor.White, RoundedCornerShape(12.dp))
                        .drawBehind {
                            drawRoundRect(
                                color = CustomColor.Black,
                                style = Stroke(
                                    width = 4f,
                                    pathEffect = PathEffect.dashPathEffect(floatArrayOf(16f, 16f), 0f)
                                ),
                                cornerRadius = CornerRadius(24f),
                            )
                        }
                        
                        .padding(Dimension.Padding.Normal),
                    verticalAlignment = Alignment.CenterVertically,
                    horizontalArrangement = Arrangement.spacedBy(
                        space = Dimension.Padding.Normal,
                        alignment = Alignment.CenterHorizontally,
                    ),
                ) {
                    IconButton(
                        onClick = {
                            if (rememberPriceToAlert > 0.0) {
                                rememberPriceToAlert -= 0.05
                            }
                            onChangePriceToAlert(rememberPriceToAlert)
                        }
                    ) {
                        Icon(
                            modifier = Modifier.size(24.dp),
                            painter = painterResource(id = R.drawable.ic_minus),
                            contentDescription = null,
                        )
                    }
                    Text(
                        text = "%.2f €".format(rememberPriceToAlert).replace('.', ','),
                        style = CustomTextStyle.TitleLarge,
                    )
                    IconButton(
                        onClick = {
                            if (rememberPriceToAlert <= 10.0) {
                                rememberPriceToAlert += 0.05
                            }
                            onChangePriceToAlert(rememberPriceToAlert)
                        }
                    ) {
                        Icon(
                            modifier = Modifier.size(24.dp),
                            painter = painterResource(id = R.drawable.ic_plus),
                            contentDescription = null
                        )
                    }
                }
            }
            item {
                Text(
                    modifier = Modifier
                        .padding(bottom = Dimension.Padding.Normal)
                        .animateItemPlacement(),
                    text = stringResource(id = R.string.list_prices_favorites),
                    style = CustomTextStyle.TitleLarge,
                )
            }
            if (uiModel.favorites.isEmpty()) {
                item {
                    Row(
                        horizontalArrangement = Arrangement.spacedBy(space = Dimension.Padding.Little),
                    ) {
                        Icon(
                            modifier = Modifier
                                .size(20.dp)
                                .offset(y = 2.dp),
                            painter = painterResource(id = R.drawable.ic_info),
                            contentDescription = null,
                        )
                        Text(
                            modifier = Modifier,
                            text = stringResource(id = R.string.list_prices_no_favorites),
                            style = CustomTextStyle.Body,
                        )
                    }
                }
            }
            items(
                count = uiModel.favorites.size,
                key = {
                    "${uiModel.favorites[it].gasStation.id}_alerts"
                },
                itemContent = {
                    ItemAlertGasStationComposable(
                        modifier = Modifier.padding(bottom = Dimension.Padding.Little),
                        item = uiModel.favorites[it],
                        onClickAlert = onClickAlert,
                        onClickFavorite = onClickFavorite,
                        onClickDetails = onClickDetails,
                        onClickNavigateMaps = onClickNavigateMaps,
                    )
                }
            )
        }
    }
}

@Composable
fun ItemAlertGasStationComposable(
    item: AlertGasStationUiModel,
    onClickAlert: (item: AlertGasStationUiModel) -> Unit,
    onClickFavorite: (item: GasStationUiModel) -> Unit,
    onClickDetails: (gasStationId: Int) -> Unit,
    onClickNavigateMaps: (NavigationMapsUiModel) -> Unit,
    modifier: Modifier = Modifier,
) {
    var rememberEnabled by remember { mutableStateOf(item.isEnabled) }
    Box(
        modifier = modifier
            .fillMaxWidth()
            .background(
                color = if (rememberEnabled) CustomColor.Green2 else CustomColor.Red2,
                shape = RoundedCornerShape(Dimension.Border.Radius),
            )
            .border(Dimension.Border.Stroke, CustomColor.Black, RoundedCornerShape(Dimension.Border.Radius)),
    ) {
        Icon(
            modifier = Modifier
                .padding(end = Dimension.Padding.Little)
                .size(24.dp)
                .align(Alignment.CenterEnd)
                .clickable {
                    rememberEnabled = rememberEnabled.not()
                    onClickAlert(item.copy(isEnabled = rememberEnabled))
                },
            painter = painterResource(id = if (rememberEnabled) R.drawable.ic_notification_turn_on else R.drawable.ic_notification_turn_off),
            contentDescription = null
        )
        ItemGasStationComposable(
            modifier = Modifier
                .padding(end = Dimension.Padding.VeryBig)
                .background(CustomColor.White, RoundedCornerShape(Dimension.Border.Radius))
                .border(Dimension.Border.Stroke, CustomColor.Black, RoundedCornerShape(Dimension.Border.Radius)),
            item = item.gasStation,
            onClickFavorite = onClickFavorite,
            onClickDetails = onClickDetails,
            onClickNavigateMaps = onClickNavigateMaps,
        )
    }
}

@Preview
@Composable
fun AlertsSuccessComposablePreview() {
    val gasStations = ImmutableList.of(
        GasStationUiModel(
            id = 5289,
            name = "ALCAMPO",
            address = "",
            brandId = 1,
            municipality = "Albacete",
            postalCode = "02006",
            latitude = 0.0,
            longitude = 0.0,
            distanceKm = "300 m",
            price = "1,589",
            updateAtText = "ayer, 11:59h",
            scoreText = "4,7",
            isFavorite = true,
            costTripTo = "67,90 €",
        ),
        GasStationUiModel(
            id = 5291,
            name = "OTHER",
            address = "",
            brandId = 17,
            municipality = "Albacete",
            postalCode = "02006",
            latitude = 0.0,
            longitude = 0.0,
            distanceKm = "5,3 km",
            price = "1,589",
            scoreText = "4,7",
            updateAtText = "ayer, 11:59h",
            isFavorite = false,
            costTripTo = "67,90 €",
        ),
    )
    val alerts = gasStations.mapIndexed { index, gasStationUiModel ->
        AlertGasStationUiModel(
            gasStation = gasStationUiModel,
            isEnabled = index == 0,
        )
    }.toImmutableList()
    AlertsSuccessComposable(
        uiModel = AlertUiModel(
            priceToAlert = 1.35,
            favorites = alerts,
            fuelTypeSelected = FuelTypeDomainEnum.DIESEL_A,
            fuelTypesToShow = FuelTypeDomainEnum.entries.toImmutableList(),
        ),
        onChangePriceToAlert = {},
        onClickFavorite = {},
        onClickAlert = {},
        onClickFuelType = {},
        onClickDetails = {},
        onClickNavigateMaps = {},
    )
}