package com.jrl.fuelprice.presentation.alerts

import com.jrl.fuelprice.domain.alerts.models.AlertDomainModel
import com.jrl.fuelprice.domain.alerts.models.AlertGasStationDomainModel
import com.jrl.fuelprice.domain.alerts.usecases.GetAlertsUseCase
import com.jrl.fuelprice.domain.alerts.usecases.UpdateStatusAlertUseCase
import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import com.jrl.fuelprice.domain.gasstations.usecases.SaveFavoriteGasStationUseCase
import com.jrl.fuelprice.domain.vehicledetails.models.VehicleDetailsDomainModel
import com.jrl.fuelprice.domain.vehicledetails.usecases.GetVehicleDetailsUseCase
import com.jrl.fuelprice.presentation.alerts.AlertUiAction.Load
import com.jrl.fuelprice.presentation.alerts.AlertUiAction.OnClickAlert
import com.jrl.fuelprice.presentation.alerts.AlertUiAction.OnClickFavorite
import com.jrl.fuelprice.presentation.alerts.AlertUiAction.OnClickFuelType
import com.jrl.fuelprice.presentation.alerts.AlertUiAction.OnLocation
import com.jrl.fuelprice.presentation.alerts.models.AlertGasStationUiModel
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.permissions.location.LocationState
import com.jrl.fuelprice.presentation.commons.viewmodel.MviViewModel
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.gasstationdetails.models.GasStationUiModel
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.collectLatest
import javax.inject.Inject

@HiltViewModel
class AlertViewModel @Inject constructor(
    private val getVehicleDetailsUseCase: GetVehicleDetailsUseCase,
    private val getAlertsUseCase: GetAlertsUseCase,
    private val updateStatusAlertUseCase: UpdateStatusAlertUseCase,
    private val saveFavoriteGasStationUseCase: SaveFavoriteGasStationUseCase,
) : MviViewModel<UiState<AlertUiModel>, AlertUiAction, AlertUiSingleEvent>() {
    
    private var vehicleDetails = VehicleDetailsDomainModel(
        consumption = 0.0,
        tank = 0,
    )
    private var fuelTypeSelected = FuelTypeDomainEnum.DIESEL_A
    private var fuelTypesToShow = FuelTypeDomainEnum.getVisibleFuels().toImmutableList()
    private var currentLocation: LocationDomainModel? = null
    private var alerts: MutableSet<AlertGasStationDomainModel> = mutableSetOf()
    private var lastIndex: Int = 0
    private var priceToAlert: Double = 1.35
    
    override fun handleAction(action: AlertUiAction) {
        when (action) {
            Load -> {
                loadVehicleDetails()
                loadData()
            }
            is AlertUiAction.OnChangePriceToAlert -> priceToAlert = action.priceToAlert
            is AlertUiAction.OnLoadMore -> {
                lastIndex = action.currentIndex
                loadData()
            }
            is OnClickFavorite -> removeFavorite(action.item)
            is OnClickAlert -> changeAlert(action.item)
            is OnClickFuelType -> onChangeFuelType(action.fuelType)
            is OnLocation -> {
                when (action.locationState) {
                    LocationState.Loading -> Unit
                    is LocationState.Success -> {
                        currentLocation = LocationDomainModel(
                            latitude = action.locationState.location.latitude,
                            longitude = action.locationState.location.longitude
                        )
                        loadData()
                    }
                    is LocationState.Error -> currentLocation = null
                }
            }
        }
    }
    
    private fun loadVehicleDetails() {
        launchIO {
            getVehicleDetailsUseCase
                .invoke()
                .onSuccess { vehicleDetails = it }
        }
    }
    
    private fun onChangeFuelType(fuelType: FuelTypeDomainEnum) {
        fuelTypeSelected = fuelType
        lastIndex = 0
        loadData()
    }
    
    private fun removeFavorite(item: GasStationUiModel) {
        launchIO {
            saveFavoriteGasStationUseCase.invoke(
                gasStationId = item.id,
                gasStationName = item.name,
                fuelType = fuelTypeSelected,
                isFavorite = false
            )
        }
    }
    
    private fun changeAlert(item: AlertGasStationUiModel) {
        launchIO {
            updateStatusAlertUseCase.invoke(
                AlertDomainModel(
                    priceCents = (priceToAlert * 1000).toInt(),     // from 1.45 to 1450, price in cents with '3 decimals'
                    gasStationId = item.gasStation.id,
                    gasStationName = item.gasStation.name,
                    fuelType = fuelTypeSelected,
                    isEnabled = item.isEnabled,
                )
            )
        }
    }
    
    private fun loadData() {
        launchDefault {
            getAlertsUseCase.invoke(
                fuelType = fuelTypeSelected,
                location = currentLocation,
            ).collectLatest {
                alerts = it.toMutableSet()
                submitUiState()
            }
        }
    }
    
    private fun submitUiState() {
        submitState(
            UiState.Success(
                AlertUiModel(
                    priceToAlert = priceToAlert,
                    favorites = alerts.toList().alertGasStationToUi(),
                    fuelTypeSelected = fuelTypeSelected,
                    fuelTypesToShow = fuelTypesToShow,
                )
            )
        )
    }
    
    private fun List<AlertGasStationDomainModel>.alertGasStationToUi() =
        this.map {
            AlertGasStationUiModel.toUi(
                gasStation = it.gasStation,
                fuelTypeSelected = fuelTypeSelected,
                vehicleDetails = vehicleDetails,
                isEnabled = it.isEnabled,
            )
        }.toImmutableList()
}
