package com.jrl.fuelprice.presentation.commons.viewmodel

import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.receiveAsFlow

sealed class UiState<out T : Any> {
    data object Loading : UiState<Nothing>()
    data class Success<T : Any>(val data: T) : UiState<T>()
    data class Error(val errorMessage: String) : UiState<Nothing>()
}

interface UiModel

interface UiAction

interface UiSingleEvent

abstract class MviViewModel<S : UiState<*>, A : UiAction, E : UiSingleEvent> : BaseViewModel() {

    private val _uiStateFlow: MutableStateFlow<S> by lazy { MutableStateFlow(initState()) }
    val uiStateFlow: StateFlow<S> = _uiStateFlow

    private val actionFlow = MutableSharedFlow<A>()

    private val _singleEventFlow = Channel<E>()
    val singleEventFlow = _singleEventFlow.receiveAsFlow()

    init {
        observeActions()
    }

    private fun observeActions() {
        launchDefault {
            actionFlow.collect { action ->
                handleAction(action)
            }
        }
    }

    open fun initState(): S = UiState.Loading as S

    abstract fun handleAction(action: A)

    fun submitAction(action: A) {
        launchDefault {
            actionFlow.emit(action)
        }
    }

    fun submitState(state: S) {
        _uiStateFlow.value = state
    }

    fun submitSingleEvent(event: E) {
        launchDefault {
            _singleEventFlow.send(event)
        }
    }
}
