package com.jrl.fuelprice.presentation.gasstationdetails

import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.GasStationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import com.jrl.fuelprice.domain.gasstations.usecases.GetGasStationByIdUseCase
import com.jrl.fuelprice.domain.gasstations.usecases.SaveFavoriteGasStationUseCase
import com.jrl.fuelprice.domain.gasstations.usecases.SetRatingGasStationUseCase
import com.jrl.fuelprice.domain.vehicledetails.models.VehicleDetailsDomainModel
import com.jrl.fuelprice.domain.vehicledetails.usecases.GetVehicleDetailsUseCase
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.permissions.location.LocationState
import com.jrl.fuelprice.presentation.commons.viewmodel.MviViewModel
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.gasstationdetails.GasStationDetailsUiAction.Load
import com.jrl.fuelprice.presentation.gasstationdetails.GasStationDetailsUiAction.OnClickFavorite
import com.jrl.fuelprice.presentation.gasstationdetails.GasStationDetailsUiAction.OnLoadMore
import com.jrl.fuelprice.presentation.gasstationdetails.GasStationDetailsUiAction.OnLocation
import com.jrl.fuelprice.presentation.gasstationdetails.GasStationDetailsUiAction.OnRatingBrand
import com.jrl.fuelprice.presentation.gasstationdetails.GasStationDetailsUiAction.OnRatingGasStation
import com.jrl.fuelprice.presentation.gasstationdetails.models.GasStationUiModel
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject

@HiltViewModel
class GasStationDetailsViewModel @Inject constructor(
    private val getGasStationByIdUseCase: GetGasStationByIdUseCase,
    private val getVehicleDetailsUseCase: GetVehicleDetailsUseCase,
    private val saveFavoriteGasStationUseCase: SaveFavoriteGasStationUseCase,
    private val setRatingGasStationUseCase: SetRatingGasStationUseCase,
) : MviViewModel<UiState<GasStationDetailsUiModel>, GasStationDetailsUiAction, GasStationDetailsUiSingleEvent>() {
    
    private var currentGasStationId: Int? = null
    private var gasStationDetailsUiModel: GasStationDetailsUiModel? = null
    private var vehicleDetails = VehicleDetailsDomainModel(
        consumption = 0.0,
        tank = 0,
    )
    private var fuelTypeSelected = FuelTypeDomainEnum.DIESEL_A
    private var location: LocationDomainModel? = null
    
    override fun handleAction(action: GasStationDetailsUiAction) {
        when (action) {
            is Load -> {
                loadVehicleDetails()
                loadGasStationDetails(action.gasStationId)
            }
            is OnLoadMore -> {
            }
            is OnClickFavorite -> saveFavorite(action.item)
            is OnLocation -> {
                when (action.locationState) {
                    LocationState.Loading -> Unit
                    is LocationState.Success -> {
                        location = LocationDomainModel(
                            latitude = action.locationState.location.latitude,
                            longitude = action.locationState.location.longitude
                        )
                        currentGasStationId?.let { loadGasStationDetails(it) }
                    }
                    is LocationState.Error -> location = null
                }
            }
            is OnRatingGasStation -> handleOnRatingGasStation(action.rating)
            is OnRatingBrand -> handleOnRatingBrand(action.rating)
        }
    }
    
    private fun loadGasStationDetails(gasStationId: Int) {
        currentGasStationId = gasStationId
        launchIO {
            getGasStationByIdUseCase.invoke(
                id = gasStationId,
                fuelType = fuelTypeSelected,
                location = location,
            ).onSuccess {
                gasStationDetailsUiModel = GasStationDetailsUiModel(
                    gasStation = GasStationUiModel.toUi(
                        gasStation = it,
                        fuelTypeSelected = fuelTypeSelected,
                        vehicleDetails = vehicleDetails,
                    ),
                    fuels = GasStationUiModel.buildFuelPricesUi(it).toImmutableList(),
                    ratingGasStationByUser = null,
                    ratingBrandByUser = null,
                )
                submitUiState()
            }.onFailure {
                // TODO
                val a = "a"
            }
        }
    }
    
    private fun loadVehicleDetails() {
        launchIO {
            getVehicleDetailsUseCase
                .invoke()
                .onSuccess { vehicleDetails = it }
        }
    }
    
    private fun saveFavorite(item: GasStationUiModel) {
        launchIO {
            saveFavoriteGasStationUseCase.invoke(
                gasStationId = item.id,
                gasStationName = item.name,
                fuelType = fuelTypeSelected,
                isFavorite = item.isFavorite,
            )
        }
        launchDefault {
            gasStationDetailsUiModel = gasStationDetailsUiModel?.gasStation?.copy(
                isFavorite = item.isFavorite
            )?.let {
                gasStationDetailsUiModel?.copy(
                    gasStation = it
                )
            }
            submitUiState()
        }
    }
    
    private fun handleOnRatingGasStation(rating: Int) {
        gasStationDetailsUiModel?.gasStation?.id?.let { gasStationId ->
            launchIO {
                setRatingGasStationUseCase.invoke(gasStationId = gasStationId, rating = rating)
            }
        }
        gasStationDetailsUiModel = gasStationDetailsUiModel?.copy(ratingGasStationByUser = rating)
        submitUiState()
    }
    
    private fun handleOnRatingBrand(rating: Int) {
        gasStationDetailsUiModel?.gasStation?.id?.let { gasStationId ->
            launchIO {
                setRatingGasStationUseCase.invoke(gasStationId = gasStationId, rating = rating)
            }
        }
        gasStationDetailsUiModel = gasStationDetailsUiModel?.copy(ratingBrandByUser = rating)
        submitUiState()
    }
    
    private fun submitUiState() {
        gasStationDetailsUiModel?.let {
            submitState(
                UiState.Success(it)
            )
        }
    }
    
    private fun List<GasStationDomainModel>.mapGasStationToUi() = this.map {
        GasStationUiModel.toUi(
            gasStation = it,
            fuelTypeSelected = fuelTypeSelected,
            vehicleDetails = vehicleDetails,
        )
    }.toImmutableList()
}
