package com.jrl.fuelprice.presentation.gasstationdetails

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.rounded.Star
import androidx.compose.material.icons.rounded.StarBorder
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import com.google.android.gms.location.LocationServices
import com.google.common.collect.ImmutableList
import com.jrl.fuelprice.R
import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.presentation.commons.composables.ErrorComposable
import com.jrl.fuelprice.presentation.commons.composables.ItemGasStationComposable
import com.jrl.fuelprice.presentation.commons.composables.LoadingComposable
import com.jrl.fuelprice.presentation.commons.permissions.location.GetLocation
import com.jrl.fuelprice.presentation.commons.permissions.location.LocationState
import com.jrl.fuelprice.presentation.commons.permissions.location.RequestLocationPermission
import com.jrl.fuelprice.presentation.commons.theme.CustomColor
import com.jrl.fuelprice.presentation.commons.theme.CustomTextStyle
import com.jrl.fuelprice.presentation.commons.theme.Dimension
import com.jrl.fuelprice.presentation.commons.utils.getStringRes
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.gasstationdetails.models.FuelPriceUiModel
import com.jrl.fuelprice.presentation.gasstationdetails.models.GasStationUiModel
import com.jrl.fuelprice.presentation.navigationmaps.models.NavigationMapsUiModel

@Composable
fun GasStationDetailsScreen(
    gasStationId: Int,
    onClickNavigateMaps: (NavigationMapsUiModel) -> Unit,
    modifier: Modifier = Modifier,
    viewModel: GasStationDetailsViewModel = hiltViewModel()
) {
    Surface(
        modifier = modifier,
        color = CustomColor.Background
    ) {
        var permissionsGranted by remember { mutableStateOf(false) }
        val fusedLocationClient = LocationServices.getFusedLocationProviderClient(LocalContext.current)
        var locationState by remember { mutableStateOf<LocationState?>(null) }
        
        LaunchedEffect(Unit) {
            viewModel.submitAction(GasStationDetailsUiAction.Load(gasStationId))
        }
        RequestLocationPermission(
            onPermissionsGranted = {
                permissionsGranted = true
            },
            onPermissionsNotGranted = {
                permissionsGranted = false
            },
        )
        viewModel.uiStateFlow.collectAsState().value.let { state ->
            when (state) {
                UiState.Loading -> LoadingComposable()
                is UiState.Success -> {
                    GasStationDetailsSuccessComposable(
                        uiModel = state.data,
                        onClickFavorite = {
                            viewModel.submitAction(GasStationDetailsUiAction.OnClickFavorite(it))
                        },
                        onClickNavigateMaps = onClickNavigateMaps,
                        onRatingGasStation = {
                            viewModel.submitAction(GasStationDetailsUiAction.OnRatingGasStation(it))
                        },
                        onRatingBrand = {
                            viewModel.submitAction(GasStationDetailsUiAction.OnRatingBrand(it))
                        },
                    )
                }
                is UiState.Error -> ErrorComposable()
            }
        }
        if (permissionsGranted) {
            GetLocation(
                fusedLocationClient = fusedLocationClient,
                onLocation = {
                    locationState = it
                    viewModel.submitAction(GasStationDetailsUiAction.OnLocation(it))
                }
            )
        }
    }
}

@Composable
fun GasStationDetailsSuccessComposable(
    uiModel: GasStationDetailsUiModel,
    onClickFavorite: (item: GasStationUiModel) -> Unit,
    onClickNavigateMaps: (NavigationMapsUiModel) -> Unit,
    onRatingGasStation: (rating: Int) -> Unit,
    onRatingBrand: (rating: Int) -> Unit,
) {
    Column(
        modifier = Modifier
            .fillMaxSize()
            .padding(Dimension.Padding.Normal),
    ) {
        ItemGasStationComposable(
            modifier = Modifier.background(CustomColor.White),
            item = uiModel.gasStation,
            onClickFavorite = onClickFavorite,
            onClickNavigateMaps = onClickNavigateMaps,
        )
        Column(
            modifier = Modifier.padding(
                vertical = Dimension.Padding.VeryBig,
                horizontal = Dimension.Padding.Big,
            )
        ) {
            uiModel.fuels.forEach {
                Row(
                    modifier = Modifier
                        .padding(bottom = Dimension.Padding.Little)
                        .fillMaxWidth()
                ) {
                    Text(
                        text = stringResource(it.type.getStringRes()),
                        style = CustomTextStyle.Body,
                    )
                    Text(
                        modifier = Modifier.weight(1f),
                        text = it.priceText,
                        style = CustomTextStyle.BodyBold.copy(
                            textAlign = TextAlign.End
                        ),
                    )
                }
            }
        }
        Text(
            text = stringResource(id = R.string.gas_station_details_rating_station),
            style = CustomTextStyle.TitleLarge,
        )
        Row {
            uiModel.ratingGasStationByUser?.let { rating ->
                repeat(5) {
                    Icon(
                        modifier = Modifier.size(48.dp),
                        imageVector = if (rating >= it + 1) {
                            Icons.Rounded.Star
                        } else {
                            Icons.Rounded.StarBorder
                        },
                        contentDescription = null,
                    )
                }
            } ?: run {
                repeat(5) {
                    IconButton(
                        modifier = Modifier.size(48.dp),
                        onClick = { onRatingGasStation(it + 1) },
                    ) {
                        Icon(
                            modifier = Modifier.fillMaxSize(),
                            imageVector = Icons.Rounded.StarBorder,
                            contentDescription = null,
                        )
                    }
                }
            }
        }
        Text(
            modifier = Modifier.padding(top = Dimension.Padding.Big),
            text = stringResource(id = R.string.gas_station_details_rating_brand),
            style = CustomTextStyle.TitleLarge,
        )
        Row {
            uiModel.ratingBrandByUser?.let { rating ->
                repeat(5) {
                    Icon(
                        modifier = Modifier.size(48.dp),
                        imageVector = if (rating >= it + 1) {
                            Icons.Rounded.Star
                        } else {
                            Icons.Rounded.StarBorder
                        },
                        contentDescription = null,
                    )
                }
            } ?: run {
                repeat(5) {
                    IconButton(
                        modifier = Modifier.size(48.dp),
                        onClick = { onRatingBrand(it + 1) },
                    ) {
                        Icon(
                            modifier = Modifier.fillMaxSize(),
                            imageVector = Icons.Rounded.StarBorder,
                            contentDescription = null,
                        )
                    }
                }
            }
        }
    }
}

@Preview
@Composable
fun GasStationDetailsSuccessComposablePreview() {
    val gasStations = ImmutableList.of(
        GasStationUiModel(
            id = 5289,
            name = "ALCAMPO",
            address = "",
            brandId = 1,
            municipality = "Albacete",
            postalCode = "02006",
            latitude = 0.0,
            longitude = 0.0,
            distanceKm = "300 m",
            price = "1,589",
            updateAtText = "ayer, 11:59h",
            scoreText = "4,7",
            isFavorite = true,
            costTripTo = "67,90 €",
        ),
        GasStationUiModel(
            id = 5291,
            name = "OTHER",
            address = "",
            brandId = 17,
            municipality = "Albacete",
            postalCode = "02006",
            latitude = 0.0,
            longitude = 0.0,
            distanceKm = "5,3 km",
            price = "1,589",
            scoreText = "4,7",
            updateAtText = "ayer, 11:59h",
            isFavorite = false,
            costTripTo = "67,90 €",
        ),
    )
    GasStationDetailsSuccessComposable(
        uiModel = GasStationDetailsUiModel(
            gasStation = gasStations.first(),
            fuels = ImmutableList.of(
                FuelPriceUiModel(FuelTypeDomainEnum.DIESEL_A, "1,339 €")
            ),
            ratingGasStationByUser = 3,
            ratingBrandByUser = null,
        ),
        onClickFavorite = {},
        onRatingGasStation = {},
        onClickNavigateMaps = {},
        onRatingBrand = {},
    )
}