package com.jrl.fuelprice.presentation.home

import androidx.annotation.DrawableRes
import androidx.annotation.StringRes
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.jrl.fuelprice.R
import com.jrl.fuelprice.presentation.alerts.AlertScreen
import com.jrl.fuelprice.presentation.commons.navigateWithCar
import com.jrl.fuelprice.presentation.commons.theme.FuelPriceTheme
import com.jrl.fuelprice.presentation.listprices.ListPricesScreen
import com.jrl.fuelprice.presentation.map.MapScreen
import com.jrl.fuelprice.presentation.navigationmaps.NavigationMapsDialog
import com.jrl.fuelprice.presentation.navigationmaps.models.NavigationMapsUiModel
import com.jrl.fuelprice.presentation.vehiclediscount.VehicleDiscountScreen

sealed class HomeScreens(
    val route: String,
    @StringRes val titleId: Int,
    @DrawableRes val iconId: Int,
) {
    data object List : HomeScreens(
        "list",
        R.string.home_list_prices_title,
        R.drawable.ic_list
    )
    
    data object Map : HomeScreens(
        "map",
        R.string.home_map_title,
        R.drawable.ic_map
    )
    
    data object Alerts : HomeScreens(
        "alerts",
        R.string.home_alerts_title,
        R.drawable.ic_notificacion_add
    )
    
    data object VehicleDiscounts : HomeScreens(
        "vehicle_discounts",
        R.string.home_vehicle_discount_title,
        R.drawable.ic_car
    )
}

@Composable
fun HomeNavigation(
    onNavigateGasStationDetails: (gasStationId: Int) -> Unit,
) {
    val context = LocalContext.current
    val navController = rememberNavController()
    var showFullSearch by remember { mutableStateOf(false) }
    
    var navigationMapsUiModel: NavigationMapsUiModel? = null
    var showDialogNavigationMaps by remember { mutableStateOf(false) }
    
    FuelPriceTheme {
        Scaffold(
            topBar = {
                HomeScreenTopBar(
                    onClickSearch = { showFullSearch = true }
                )
            },
            bottomBar = {
                HomeBottomBar(
                    navController = navController,
                    items = listOf(
                        HomeScreens.List,
                        HomeScreens.Map,
                        HomeScreens.Alerts,
                        HomeScreens.VehicleDiscounts,
                    ),
                )
            },
            content = { innerPadding ->
                NavHost(
                    navController = navController,
                    startDestination = HomeScreens.List.route,
                    modifier = Modifier.padding(innerPadding)
                ) {
                    composable(HomeScreens.List.route) {
                        ListPricesScreen(
                            showFullSearch = showFullSearch,
                            onDismissSearchDialog = { showFullSearch = false },
                            onClickDetails = onNavigateGasStationDetails,
                            onClickNavigateMaps = {
                                navigationMapsUiModel = it
                                showDialogNavigationMaps = true
                            }
                        )
                    }
                    composable(HomeScreens.Map.route) {
                        MapScreen(
                            onClickDetails = onNavigateGasStationDetails,
                        )
                    }
                    composable(HomeScreens.Alerts.route) {
                        AlertScreen(
                            onClickDetails = onNavigateGasStationDetails,
                            onClickNavigateMaps = {
                                navigationMapsUiModel = it
                                showDialogNavigationMaps = true
                            }
                        )
                    }
                    composable(HomeScreens.VehicleDiscounts.route) { VehicleDiscountScreen() }
                }
            },
        )
        if (showDialogNavigationMaps && navigationMapsUiModel != null) {
            NavigationMapsDialog(
                navigationMapsUiModel = navigationMapsUiModel!!,
                onAccept = { navigateWithCar(context = context, latitude = it.latitude, longitude = it.longitude) },
                onCancel = { showDialogNavigationMaps = false }
            )
        }
    }
}