package com.jrl.fuelprice.presentation.listprices

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.ExperimentalLayoutApi
import androidx.compose.foundation.layout.FlowRow
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material3.Icon
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.runtime.snapshotFlow
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import com.google.android.gms.location.LocationServices
import com.google.common.collect.ImmutableList
import com.jrl.fuelprice.R
import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.RadioDomainEnum
import com.jrl.fuelprice.presentation.commons.composables.ErrorComposable
import com.jrl.fuelprice.presentation.commons.composables.ItemGasStationComposable
import com.jrl.fuelprice.presentation.commons.composables.LoadingComposable
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.permissions.location.GetLocation
import com.jrl.fuelprice.presentation.commons.permissions.location.LocationState
import com.jrl.fuelprice.presentation.commons.permissions.location.RequestLocationPermission
import com.jrl.fuelprice.presentation.commons.theme.CustomColor
import com.jrl.fuelprice.presentation.commons.theme.CustomTextStyle
import com.jrl.fuelprice.presentation.commons.theme.Dimension
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.gasstationdetails.models.GasStationUiModel
import com.jrl.fuelprice.presentation.listprices.composables.FuelTypeDropdownMenu
import com.jrl.fuelprice.presentation.listprices.composables.RadioDropdownMenu
import com.jrl.fuelprice.presentation.navigationmaps.models.NavigationMapsUiModel
import com.jrl.fuelprice.presentation.search.SearchScreen
import kotlinx.coroutines.flow.filter

@Composable
fun ListPricesScreen(
    showFullSearch: Boolean,
    onDismissSearchDialog: () -> Unit,
    onClickDetails: (gasStationId: Int) -> Unit,
    onClickNavigateMaps: (NavigationMapsUiModel) -> Unit,
    modifier: Modifier = Modifier,
    viewModel: ListPricesViewModel = hiltViewModel()
) {
    Surface(
        modifier = modifier,
        color = CustomColor.Background
    ) {
        var permissionsGranted by remember { mutableStateOf(false) }
        val fusedLocationClient = LocationServices.getFusedLocationProviderClient(LocalContext.current)
        var locationState by remember { mutableStateOf<LocationState?>(null) }
        
        LaunchedEffect(Unit) {
            viewModel.submitAction(ListPricesUiAction.Load)
        }
        RequestLocationPermission(
            onPermissionsGranted = {
                permissionsGranted = true
            },
            onPermissionsNotGranted = {
                permissionsGranted = false
            },
        )
        viewModel.uiStateFlow.collectAsState().value.let { state ->
            when (state) {
                UiState.Loading -> LoadingComposable()
                is UiState.Success -> {
                    ListPricesSuccessComposable(
                        uiModel = state.data,
                        onLoadMore = {
                            viewModel.submitAction(ListPricesUiAction.OnLoadMore(it))
                        },
                        onClickFavorite = {
                            viewModel.submitAction(ListPricesUiAction.OnClickFavorite(it))
                        },
                        onClickFuelType = {
                            viewModel.submitAction(ListPricesUiAction.OnClickFuelType(it))
                        },
                        onClickRadio = {
                            viewModel.submitAction(ListPricesUiAction.OnClickRadio(it))
                        },
                        onClickDetails = onClickDetails,
                        onClickNavigateMaps = onClickNavigateMaps,
                    )
                }
                is UiState.Error -> ErrorComposable()
            }
        }
        if (permissionsGranted) {
            GetLocation(
                fusedLocationClient = fusedLocationClient,
                onLocation = {
                    locationState = it
                    viewModel.submitAction(ListPricesUiAction.OnLocation(it))
                }
            )
        }
        AnimatedVisibility(visible = showFullSearch) {
            SearchScreen(
                onClickPlace = {
                    viewModel.submitAction(ListPricesUiAction.OnClickPlace(it))
                    onDismissSearchDialog()
                },
                onDismissSearchDialog = onDismissSearchDialog,
            )
        }
    }
}

@OptIn(
    ExperimentalFoundationApi::class,
    ExperimentalLayoutApi::class
)
@Composable
fun ListPricesSuccessComposable(
    uiModel: ListPricesUiModel,
    onLoadMore: (currentIndex: Int) -> Unit,
    onClickFavorite: (item: GasStationUiModel) -> Unit,
    onClickFuelType: (fuelType: FuelTypeDomainEnum) -> Unit,
    onClickRadio: (radio: RadioDomainEnum) -> Unit,
    onClickDetails: (gasStationId: Int) -> Unit,
    onClickNavigateMaps: (NavigationMapsUiModel) -> Unit,
) {
    val scrollState = rememberLazyListState()
    LaunchedEffect(scrollState) {
        snapshotFlow { scrollState.firstVisibleItemIndex }.filter { index -> index > uiModel.gasStations.size - 10 } // Load more when 10 items from the end
            .collect { onLoadMore(uiModel.gasStations.size) }
    }
    uiModel.gasStations.takeIf { it.isNotEmpty() }?.let {
        Column(
            modifier = Modifier
                .fillMaxSize()
                .padding(Dimension.Padding.Normal),
        ) {
            FlowRow(
                modifier = Modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.spacedBy(space = Dimension.Padding.Little),
            ) {
                FuelTypeDropdownMenu(
                    itemSelected = uiModel.fuelTypeSelected,
                    items = uiModel.fuelTypesToShow,
                    onClick = onClickFuelType,
                )
                RadioDropdownMenu(
                    itemSelected = uiModel.radioSelected,
                    items = uiModel.radiosToShow,
                    onClick = onClickRadio,
                )
            }
            LazyColumn(
                modifier = Modifier.fillMaxSize(),
                state = scrollState,
            ) {
                item {
                    Text(
                        modifier = Modifier
                            .padding(bottom = Dimension.Padding.Normal)
                            .animateItemPlacement(),
                        text = stringResource(id = R.string.list_prices_favorites),
                        style = CustomTextStyle.TitleLarge,
                    )
                }
                if (uiModel.favorites.isEmpty()) {
                    item {
                        Row(
                            horizontalArrangement = Arrangement.spacedBy(space = Dimension.Padding.Little),
                        ) {
                            Icon(
                                modifier = Modifier
                                    .size(20.dp)
                                    .offset(y = 2.dp),
                                painter = painterResource(id = R.drawable.ic_info),
                                contentDescription = null,
                            )
                            Text(
                                modifier = Modifier,
                                text = stringResource(id = R.string.list_prices_no_favorites),
                                style = CustomTextStyle.Body,
                            )
                        }
                    }
                }
                items(
                    count = uiModel.favorites.size,
                    key = {
                        "${uiModel.favorites[it].id}_favorites"
                    },
                    itemContent = {
                        ItemGasStationComposable(
                            modifier = Modifier
                                .background(CustomColor.White)
                                .padding(bottom = Dimension.Padding.Little),
                            item = uiModel.favorites[it],
                            onClickFavorite = onClickFavorite,
                            onClickDetails = onClickDetails,
                            onClickNavigateMaps = onClickNavigateMaps,
                        )
                    }
                )
                item {
                    Text(
                        modifier = Modifier.padding(
                            top = Dimension.Padding.Big,
                            bottom = Dimension.Padding.Normal,
                        ),
                        text = stringResource(id = R.string.list_prices_rest),
                        style = CustomTextStyle.TitleLarge,
                    )
                }
                
                items(
                    count = uiModel.gasStations.size,
                    key = { uiModel.gasStations[it].id },
                    itemContent = {
                        ItemGasStationComposable(
                            modifier = Modifier
                                .background(CustomColor.White)
                                .padding(bottom = Dimension.Padding.Little)
                                .animateItemPlacement(),
                            item = uiModel.gasStations[it],
                            onClickFavorite = onClickFavorite,
                            onClickDetails = onClickDetails,
                            onClickNavigateMaps = onClickNavigateMaps,
                        )
                    }
                )
            }
        }
    } ?: run {
        Column(
            modifier = Modifier
                .padding(Dimension.Padding.VeryBig)
                .fillMaxSize(),
            verticalArrangement = Arrangement.Center,
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            Text(
                modifier = Modifier.padding(top = Dimension.Padding.Normal),
                text = stringResource(id = R.string.list_empty),
                style = CustomTextStyle.BodyBold,
            )
        }
    }
}

@Preview
@Composable
private fun ListPricesSuccessComposablePreview() {
    val gasStations = ImmutableList.of(
        GasStationUiModel(
            id = 5289,
            name = "ALCAMPO",
            address = "",
            brandId = 1,
            municipality = "Albacete",
            postalCode = "02006",
            latitude = 0.0,
            longitude = 0.0,
            distanceKm = "300 m",
            price = "1,589",
            updateAtText = "ayer, 11:59h",
            scoreText = "4,7",
            isFavorite = true,
            costTripTo = "67,90 €",
        ),
        GasStationUiModel(
            id = 5291,
            name = "OTHER",
            address = "",
            brandId = 17,
            municipality = "Albacete",
            postalCode = "02006",
            latitude = 0.0,
            longitude = 0.0,
            distanceKm = "5,3 km",
            price = "1,589",
            scoreText = "4,7",
            updateAtText = "ayer, 11:59h",
            isFavorite = false,
            costTripTo = "67,90 €",
        ),
    )
    ListPricesSuccessComposable(
        uiModel = ListPricesUiModel(
            fuelTypeSelected = FuelTypeDomainEnum.DIESEL_A,
            fuelTypesToShow = FuelTypeDomainEnum.entries.toImmutableList(),
            radioSelected = RadioDomainEnum.TWENTY_FIVE_KM,
            radiosToShow = RadioDomainEnum.entries.toImmutableList(),
            favorites = ImmutableList.of(gasStations.first()),
            gasStations = gasStations,
        ),
        onLoadMore = {},
        onClickFavorite = {},
        onClickFuelType = {},
        onClickRadio = {},
        onClickDetails = {},
        onClickNavigateMaps = {},
    )
}

@Preview
@Composable
private fun ListPricesSuccessComposablePreviewEmpty() {
    ListPricesSuccessComposable(
        uiModel = ListPricesUiModel(
            fuelTypeSelected = FuelTypeDomainEnum.DIESEL_A,
            fuelTypesToShow = FuelTypeDomainEnum.entries.toImmutableList(),
            radioSelected = RadioDomainEnum.TWENTY_FIVE_KM,
            radiosToShow = RadioDomainEnum.entries.toImmutableList(),
            favorites = ImmutableList.of(),
            gasStations = ImmutableList.of(),
        ),
        onLoadMore = {},
        onClickFavorite = {},
        onClickFuelType = {},
        onClickRadio = {},
        onClickDetails = {},
        onClickNavigateMaps = {},
    )
}