package com.jrl.fuelprice.presentation.listprices

import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.GasStationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.RadioDomainEnum
import com.jrl.fuelprice.domain.gasstations.usecases.GetGasStationsUseCase
import com.jrl.fuelprice.domain.gasstations.usecases.SaveFavoriteGasStationUseCase
import com.jrl.fuelprice.domain.places.models.PlaceDomainModel
import com.jrl.fuelprice.domain.places.usecases.GetLocationFromPlaceIdUseCase
import com.jrl.fuelprice.domain.vehicledetails.models.VehicleDetailsDomainModel
import com.jrl.fuelprice.domain.vehicledetails.usecases.GetVehicleDetailsUseCase
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.permissions.location.LocationState
import com.jrl.fuelprice.presentation.commons.viewmodel.MviViewModel
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.gasstationdetails.models.GasStationUiModel
import com.jrl.fuelprice.presentation.listprices.ListPricesUiAction.Load
import com.jrl.fuelprice.presentation.listprices.ListPricesUiAction.OnClickFavorite
import com.jrl.fuelprice.presentation.listprices.ListPricesUiAction.OnClickFuelType
import com.jrl.fuelprice.presentation.listprices.ListPricesUiAction.OnClickPlace
import com.jrl.fuelprice.presentation.listprices.ListPricesUiAction.OnClickRadio
import com.jrl.fuelprice.presentation.listprices.ListPricesUiAction.OnLoadMore
import com.jrl.fuelprice.presentation.listprices.ListPricesUiAction.OnLocation
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.collectLatest
import javax.inject.Inject

@HiltViewModel
class ListPricesViewModel @Inject constructor(
    private val getVehicleDetailsUseCase: GetVehicleDetailsUseCase,
    private val getGasStationsUseCase: GetGasStationsUseCase,
    private val saveFavoriteGasStationUseCase: SaveFavoriteGasStationUseCase,
    private val getLocationFromPlaceIdUseCase: GetLocationFromPlaceIdUseCase,
) : MviViewModel<UiState<ListPricesUiModel>, ListPricesUiAction, ListPricesUiSingleEvent>() {
    
    private var vehicleDetails = VehicleDetailsDomainModel(
        consumption = 0.0,
        tank = 0,
    )
    private var fuelTypeSelected = FuelTypeDomainEnum.DIESEL_A
    private var fuelTypesToShow = FuelTypeDomainEnum.getVisibleFuels().toImmutableList()
    private var radioSelected = RadioDomainEnum.TWENTY_FIVE_KM
    private var radiosToShow = RadioDomainEnum.entries.toImmutableList()
    
    private var gasStations: MutableList<GasStationDomainModel> = mutableListOf()
    private var location: LocationDomainModel? = null
    private var lastIndex: Int = 0
    
    override fun handleAction(action: ListPricesUiAction) {
        when (action) {
            Load -> {
                loadVehicleDetails()
                loadGasStations()
            }
            is OnLoadMore -> {
                lastIndex = action.currentIndex
                loadGasStations()
            }
            is OnClickFavorite -> saveFavorite(action.item)
            is OnClickFuelType -> onChangeFuelType(action.fuelType)
            is OnClickRadio -> onChangeRadio(action.radio)
            is OnLocation -> {
                when (action.locationState) {
                    LocationState.Loading -> Unit
                    is LocationState.Success -> {
                        location = LocationDomainModel(
                            latitude = action.locationState.location.latitude,
                            longitude = action.locationState.location.longitude
                        )
                        loadGasStations()
                    }
                    is LocationState.Error -> location = null
                }
                loadGasStations()
            }
            is OnClickPlace -> {
                getLocationFromPlaceId(action.place)
            }
        }
    }
    
    private fun getLocationFromPlaceId(place: PlaceDomainModel) {
        launchIO {
            getLocationFromPlaceIdUseCase.invoke(place.id)
                .onSuccess {
                    location = it
                    loadGasStations()
                }
                .onFailure { }
        }
    }
    
    private fun loadVehicleDetails() {
        launchIO {
            getVehicleDetailsUseCase
                .invoke()
                .onSuccess { vehicleDetails = it }
        }
    }
    
    private fun onChangeFuelType(fuelType: FuelTypeDomainEnum) {
        fuelTypeSelected = fuelType
        lastIndex = 0
        loadGasStations()
    }
    
    private fun onChangeRadio(radio: RadioDomainEnum) {
        radioSelected = radio
        lastIndex = 0
        loadGasStations()
    }
    
    private fun saveFavorite(item: GasStationUiModel) {
        launchIO {
            saveFavoriteGasStationUseCase.invoke(
                gasStationId = item.id,
                gasStationName = item.name,
                fuelType = fuelTypeSelected,
                isFavorite = item.isFavorite,
            )
        }
        launchDefault {
            gasStations.find { it.id == item.id }?.isFavorite = item.isFavorite
            submitUiState()
        }
    }
    
    private fun loadGasStations() {
        launchDefault {
            getGasStationsUseCase.invoke(
                fuelType = fuelTypeSelected,
                radio = radioSelected,
                fromIndex = lastIndex,
                location = location,
            ).collectLatest {
                if (lastIndex == 0) {
                    gasStations = it.toMutableList()
                } else {
                    gasStations.addAll(it)
                }
                submitUiState()
            }
        }
    }
    
    private fun submitUiState() {
        submitState(
            UiState.Success(
                ListPricesUiModel(
                    fuelTypeSelected = fuelTypeSelected,
                    fuelTypesToShow = fuelTypesToShow,
                    radioSelected = radioSelected,
                    radiosToShow = radiosToShow,
                    favorites = gasStations.filter { it.isFavorite }.mapGasStationToUi(),
                    gasStations = gasStations.mapGasStationToUi(),
                )
            )
        )
    }
    
    private fun List<GasStationDomainModel>.mapGasStationToUi() = this.map {
        GasStationUiModel.toUi(
            gasStation = it,
            fuelTypeSelected = fuelTypeSelected,
            vehicleDetails = vehicleDetails,
        )
    }.toImmutableList()
}
