package com.jrl.fuelprice.presentation.map

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.rounded.LocalGasStation
import androidx.compose.material3.Icon
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.runtime.snapshotFlow
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import com.google.android.gms.location.LocationServices
import com.google.android.gms.maps.model.CameraPosition
import com.google.android.gms.maps.model.LatLng
import com.google.common.collect.ImmutableList
import com.google.maps.android.compose.GoogleMap
import com.google.maps.android.compose.MapProperties
import com.google.maps.android.compose.MapType
import com.google.maps.android.compose.MapUiSettings
import com.google.maps.android.compose.MarkerComposable
import com.google.maps.android.compose.MarkerState
import com.google.maps.android.compose.rememberCameraPositionState
import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import com.jrl.fuelprice.presentation.commons.composables.ErrorComposable
import com.jrl.fuelprice.presentation.commons.composables.LoadingComposable
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.permissions.location.GetLocation
import com.jrl.fuelprice.presentation.commons.permissions.location.LocationState
import com.jrl.fuelprice.presentation.commons.permissions.location.RequestLocationPermission
import com.jrl.fuelprice.presentation.commons.theme.CustomColor
import com.jrl.fuelprice.presentation.commons.theme.CustomTextStyle
import com.jrl.fuelprice.presentation.commons.theme.Dimension
import com.jrl.fuelprice.presentation.commons.utils.getBrandImg
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.map.models.MapGasStationUiModel
import com.jrl.fuelprice.presentation.map.models.PriceLevelUiEnum

@Composable
fun MapScreen(
    onClickDetails: (gasStationId: Int) -> Unit,
    modifier: Modifier = Modifier,
    viewModel: MapViewModel = hiltViewModel()
) {
    Surface(
        modifier = modifier,
        color = CustomColor.Background
    ) {
        var permissionsGranted by remember { mutableStateOf(false) }
        val fusedLocationClient = LocationServices.getFusedLocationProviderClient(LocalContext.current)
        var locationState by remember { mutableStateOf<LocationState?>(null) }
        
        LaunchedEffect(Unit) {
            viewModel.submitAction(MapUiAction.Load)
        }
        RequestLocationPermission(
            onPermissionsGranted = {
                permissionsGranted = true
            },
            onPermissionsNotGranted = {
            
            },
        )
        viewModel.uiStateFlow.collectAsState().value.let { state ->
            when (state) {
                UiState.Loading -> LoadingComposable()
                is UiState.Success -> {
                    MapSuccessComposable(
                        uiModel = state.data,
                        onCameraMove = {
                            viewModel.submitAction(MapUiAction.OnCameraMove(it))
                        },
                        onClickDetails = onClickDetails,
                    )
                }
                is UiState.Error -> ErrorComposable()
            }
            
        }
        if (permissionsGranted) {
            GetLocation(
                fusedLocationClient = fusedLocationClient,
                onLocation = {
                    locationState = it
                    viewModel.submitAction(MapUiAction.OnLocation(it))
                }
            )
        }
    }
}

@Composable
fun MapSuccessComposable(
    uiModel: MapUiModel,
    onCameraMove: (location: LocationDomainModel) -> Unit,
    onClickDetails: (gasStationId: Int) -> Unit,
) {
    val mapProperties = MapProperties(
        isBuildingEnabled = false,
        isIndoorEnabled = false,
        isMyLocationEnabled = true,
        isTrafficEnabled = false,
        mapType = MapType.NORMAL,
    )
    val mapUiSettings = MapUiSettings(
        compassEnabled = true,
        mapToolbarEnabled = true,
        myLocationButtonEnabled = true,
        rotationGesturesEnabled = true,
        scrollGesturesEnabled = true,
        tiltGesturesEnabled = true,
        zoomControlsEnabled = true,
        zoomGesturesEnabled = true,
    )
    val cameraPositionState = rememberCameraPositionState {
        position = CameraPosition.fromLatLngZoom(
            LatLng(
                uiModel.currentLocation?.latitude ?: 0.0,
                uiModel.currentLocation?.longitude ?: 0.0,
            ),
            12f,
        )
    }
    LaunchedEffect(cameraPositionState) {
        snapshotFlow { cameraPositionState.position }
            .collect {
                onCameraMove(LocationDomainModel(it.target.latitude, it.target.longitude))
            }
    }
    var showMap by remember { mutableStateOf(false) }
    GoogleMap(
        modifier = Modifier.fillMaxSize(),
        properties = mapProperties,
        uiSettings = mapUiSettings,
        cameraPositionState = cameraPositionState,
        onMapLongClick = {},
        onMapLoaded = {
            showMap = true
        },
        onMyLocationClick = {},
        onPOIClick = {},
        contentPadding = PaddingValues(Dimension.Padding.Normal),
    ) {
        uiModel.gasStations.forEach { gasStation ->
            MarkerComposable(
                state = MarkerState(position = LatLng(gasStation.latitude, gasStation.longitude)),
                onClick = {
                    onClickDetails(gasStation.id)
                    true
                },
            ) {
                Column(
                    modifier = Modifier.background(CustomColor.Transparent),
                    verticalArrangement = Arrangement.Center,
                    horizontalAlignment = Alignment.CenterHorizontally,
                ) {
                    Row(
                        verticalAlignment = Alignment.CenterVertically,
                        horizontalArrangement = Arrangement.spacedBy(Dimension.Padding.VeryLittle),
                    ) {
                        Text(
                            modifier = Modifier,
                            text = "${gasStation.price} €",
                            style = CustomTextStyle.TitleLarge.copy(
                                color = when (gasStation.priceLevel) {
                                    PriceLevelUiEnum.LOW -> CustomColor.Green3
                                    PriceLevelUiEnum.MEDIUM -> CustomColor.Orange
                                    PriceLevelUiEnum.HIGH -> CustomColor.Red3
                                },
                            ),
                        )
                    }
                    val modifierLogo = Modifier
                        .size(36.dp)
                        .border(
                            border = BorderStroke(
                                width = Dimension.Border.Stroke,
                                color = when (gasStation.priceLevel) {
                                    PriceLevelUiEnum.LOW -> CustomColor.Green3
                                    PriceLevelUiEnum.MEDIUM -> CustomColor.Orange
                                    PriceLevelUiEnum.HIGH -> CustomColor.Red3
                                },
                            ),
                            shape = CircleShape,
                        )
                    getBrandImg(gasStation.brandId)?.let { resImg ->
                        Image(
                            modifier = modifierLogo,
                            painter = painterResource(resImg),
                            contentDescription = null,
                        )
                    } ?: run {
                        Icon(
                            modifier = modifierLogo,
                            imageVector = Icons.Rounded.LocalGasStation,
                            contentDescription = null,
                        )
                    }
                }
            }
        }
    }
    if (showMap.not()) {
        LoadingComposable()
    }
}

@Preview
@Composable
fun MapSuccessComposablePreview() {
    val gasStations = ImmutableList.of(
        MapGasStationUiModel(
            id = 5289,
            name = "ALCAMPO",
            brandId = 1,
            latitude = 0.0,
            longitude = 0.0,
            price = "1,589",
            priceLevel = PriceLevelUiEnum.MEDIUM,
        ),
        MapGasStationUiModel(
            id = 5291,
            name = "ALCAMPO",
            brandId = 1,
            latitude = 0.0,
            longitude = 0.0,
            price = "1,589",
            priceLevel = PriceLevelUiEnum.MEDIUM,
        ),
    )
    MapSuccessComposable(
        uiModel = MapUiModel(
            fuelTypeSelected = FuelTypeDomainEnum.DIESEL_A,
            fuelTypesToShow = FuelTypeDomainEnum.entries.toImmutableList(),
            currentLocation = LocationDomainModel(
                latitude = 0.0,
                longitude = 0.0,
            ),
            gasStations = gasStations,
        ),
        onCameraMove = {},
        onClickDetails = {},
    )
}