package com.jrl.fuelprice.presentation.map

import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.GasStationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import com.jrl.fuelprice.domain.gasstations.models.RadioDomainEnum
import com.jrl.fuelprice.domain.gasstations.usecases.GetGasStationsUseCase
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.permissions.location.LocationState
import com.jrl.fuelprice.presentation.commons.viewmodel.MviViewModel
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.map.MapUiAction.Load
import com.jrl.fuelprice.presentation.map.MapUiAction.OnCameraMove
import com.jrl.fuelprice.presentation.map.MapUiAction.OnLocation
import com.jrl.fuelprice.presentation.map.models.MapGasStationUiModel
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.collectLatest
import javax.inject.Inject

@HiltViewModel
class MapViewModel @Inject constructor(
    private val getGasStationsUseCase: GetGasStationsUseCase,
) : MviViewModel<UiState<MapUiModel>, MapUiAction, MapUiSingleEvent>() {
    
    private var fuelTypeSelected = FuelTypeDomainEnum.DIESEL_A
    private var fuelTypesToShow = FuelTypeDomainEnum.entries.toImmutableList()
    private var currentLocation: LocationDomainModel? = null
    private var gasStations: MutableSet<GasStationDomainModel> = mutableSetOf()
    private var lastIndex: Int = 0
    
    override fun handleAction(action: MapUiAction) {
        when (action) {
            Load -> loadData()
            is OnLocation -> handledOnLocationGranted(action.locationState)
            is OnCameraMove -> handledOnCameraMove(action.location)
        }
    }
    
    private fun loadData() {
        launchDefault {
            getGasStationsUseCase.invoke(
                fuelType = fuelTypeSelected,
                radio = RadioDomainEnum.FIVE_KM,
                fromIndex = lastIndex,
                location = currentLocation,
            ).collectLatest {
                gasStations.addAll(it)
                submitUiState()
            }
        }
    }
    
    private fun handledOnLocationGranted(locationState: LocationState) {
        when (locationState) {
            LocationState.Loading -> Unit
            is LocationState.Success -> {
                currentLocation = LocationDomainModel(
                    latitude = locationState.location.latitude,
                    longitude = locationState.location.longitude
                )
            }
            is LocationState.Error -> currentLocation = null
        }
        loadData()
    }
    
    private fun handledOnCameraMove(location: LocationDomainModel) {
        currentLocation = LocationDomainModel(
            latitude = location.latitude,
            longitude = location.longitude
        )
        loadData()
    }
    
    private fun submitUiState() {
        submitState(
            UiState.Success(
                MapUiModel(
                    fuelTypeSelected = fuelTypeSelected,
                    fuelTypesToShow = fuelTypesToShow,
                    currentLocation = currentLocation,
                    gasStations = gasStations.toList().mapGasStationToUi(),
                )
            )
        )
    }
    
    private fun List<GasStationDomainModel>.mapGasStationToUi() =
        this.map {
            MapGasStationUiModel.toUi(
                gasStation = it,
                fuelTypeSelected = fuelTypeSelected,
                minPrice = this.minOf {
                    when (fuelTypeSelected) {
                        FuelTypeDomainEnum.BIODIESEL -> it.prices.biodiesel ?: 0.0
                        FuelTypeDomainEnum.DIESEL_A -> it.prices.dieselA ?: 0.0
                        FuelTypeDomainEnum.DIESEL_B -> it.prices.dieselB ?: 0.0
                        FuelTypeDomainEnum.DIESEL_PREMIUM -> it.prices.dieselPremium ?: 0.0
                        FuelTypeDomainEnum.GASOLINE_95_E5 -> it.prices.gasoline95e5 ?: 0.0
                        FuelTypeDomainEnum.GASOLINE_95_E5_PREMIUM -> it.prices.gasoline95e5Premium ?: 0.0
                        FuelTypeDomainEnum.GASOLINE_95_E10 -> it.prices.gasoline95e10 ?: 0.0
                        FuelTypeDomainEnum.GASOLINE_98_E5 -> it.prices.gasoline98e5 ?: 0.0
                        FuelTypeDomainEnum.GASOLINE_98_E10 -> it.prices.gasoline98e10 ?: 0.0
                        FuelTypeDomainEnum.HYDROGEN -> it.prices.hydrogen ?: 0.0
                    }
                }
            )
        }.toImmutableList()
}
