package com.jrl.fuelprice.presentation.search

import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.Dialog
import androidx.compose.ui.window.DialogProperties
import androidx.hilt.navigation.compose.hiltViewModel
import com.google.common.collect.ImmutableList
import com.jrl.fuelprice.domain.places.models.PlaceDomainModel
import com.jrl.fuelprice.presentation.commons.composables.CustomTextFieldComposable
import com.jrl.fuelprice.presentation.commons.composables.ErrorComposable
import com.jrl.fuelprice.presentation.commons.composables.LoadingComposable
import com.jrl.fuelprice.presentation.commons.theme.CustomColor
import com.jrl.fuelprice.presentation.commons.theme.Dimension
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState

@Composable
fun SearchScreen(
    onClickPlace: (PlaceDomainModel) -> Unit,
    onDismissSearchDialog: () -> Unit,
    viewModel: SearchViewModel = hiltViewModel(),
) {
    Dialog(
        onDismissRequest = onDismissSearchDialog,
        properties = DialogProperties(
            usePlatformDefaultWidth = false,
            decorFitsSystemWindows = true,
        ),
    ) {
        LaunchedEffect(Unit) {
            viewModel.submitAction(SearchUiAction.Load)
        }
        viewModel.uiStateFlow.collectAsState().value.let { state ->
            when (state) {
                UiState.Loading -> LoadingComposable()
                is UiState.Success -> {
                    SearchSuccessComposable(
                        uiModel = state.data,
                        onSearch = {
                            viewModel.submitAction(SearchUiAction.OnSearch(it))
                        },
                        onClickPlace = onClickPlace,
                    )
                }
                is UiState.Error -> ErrorComposable()
            }
            
        }
    }
}

@Composable
fun SearchSuccessComposable(
    modifier: Modifier = Modifier,
    uiModel: SearchUiModel,
    onSearch: (String) -> Unit,
    onClickPlace: (PlaceDomainModel) -> Unit,
) {
    Column(
        modifier = modifier
            .background(CustomColor.Black50)
            .padding(Dimension.Padding.Normal)
            .fillMaxSize(),
        verticalArrangement = Arrangement.spacedBy(Dimension.Padding.Normal),
    ) {
        CustomTextFieldComposable(
            modifier = Modifier,
            height = 48.dp,
            cornerRadius = 120.dp,
            value = uiModel.searchText,
            onValueChange = onSearch,
        )
        uiModel.places.takeIf { it.isNotEmpty() }?.let { places ->
            SearchListComposable(
                places = places,
                onClickPlace = onClickPlace,
            )
        }
    }
}

@Composable
fun SearchListComposable(
    modifier: Modifier = Modifier,
    places: ImmutableList<PlaceDomainModel>,
    onClickPlace: (PlaceDomainModel) -> Unit,
) {
    Column(
        modifier = modifier
            .fillMaxWidth()
            .background(CustomColor.White, RoundedCornerShape(24.dp))
            .border(Dimension.Border.Stroke, CustomColor.Black, RoundedCornerShape(24.dp))
            .padding(Dimension.Padding.Normal),
        verticalArrangement = Arrangement.spacedBy(Dimension.Padding.Little),
    ) {
        places.forEachIndexed { index, place ->
            SearchListItemComposable(
                place = place,
                onClick = { onClickPlace(place) },
            )
            if (index < places.size - 1) {
                HorizontalDivider()
            }
        }
    }
}

@Composable
fun SearchListItemComposable(
    place: PlaceDomainModel,
    onClick: () -> Unit,
) {
    Text(
        modifier = Modifier
            .fillMaxWidth()
            .padding(vertical = Dimension.Padding.VeryLittle)
            .clickable { onClick() },
        text = place.fullName,
    )
}

@Preview
@Composable
fun SearchSuccessComposablePreview() {
    SearchSuccessComposable(
        uiModel = SearchUiModel(
            searchText = "tol",
            places = ImmutableList.of(
                PlaceDomainModel(
                    id = "1",
                    fullName = "Toledo, Spain",
                ),
                PlaceDomainModel(
                    id = "1",
                    fullName = "Toledo, Spain",
                ),
                PlaceDomainModel(
                    id = "1",
                    fullName = "Toledo, Spain",
                ),
            ),
        ),
        onSearch = {},
        onClickPlace = {},
    )
}