package com.jrl.fuelprice.presentation.vehiclediscount

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import com.google.common.collect.ImmutableList
import com.jrl.fuelprice.R
import com.jrl.fuelprice.domain.discounts.models.DiscountDomainModel
import com.jrl.fuelprice.domain.discounts.models.DiscountTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.BrandDomainModel
import com.jrl.fuelprice.presentation.commons.composables.CustomTextFieldComposable
import com.jrl.fuelprice.presentation.commons.composables.ErrorComposable
import com.jrl.fuelprice.presentation.commons.composables.LoadingComposable
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.theme.CustomColor
import com.jrl.fuelprice.presentation.commons.theme.CustomTextStyle
import com.jrl.fuelprice.presentation.commons.theme.Dimension
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState

@Composable
fun VehicleDiscountScreen(
    modifier: Modifier = Modifier,
    viewModel: VehicleDiscountViewModel = hiltViewModel()
) {
    Surface(
        modifier = modifier,
        color = CustomColor.Background
    ) {
        LaunchedEffect(Unit) {
            viewModel.submitAction(VehicleDiscountUiAction.Load)
        }
        viewModel.uiStateFlow.collectAsState().value.let { state ->
            when (state) {
                UiState.Loading -> LoadingComposable()
                is UiState.Success -> {
                    VehicleDiscountSuccessComposable(
                        uiModel = state.data,
                        onChangeConsumption = {
                            viewModel.submitAction(VehicleDiscountUiAction.OnChangeConsumption(it))
                        },
                        onChangeTank = {
                            viewModel.submitAction(VehicleDiscountUiAction.OnChangeTank(it))
                        },
                        onAddDiscount = {
                            viewModel.submitAction(VehicleDiscountUiAction.OnAddDiscount(it))
                        },
                        onRemoveDiscount = {
                            viewModel.submitAction(VehicleDiscountUiAction.OnRemoveDiscount(it))
                        },
                    )
                }
                is UiState.Error -> ErrorComposable()
            }
            
        }
    }
}

@Composable
fun VehicleDiscountSuccessComposable(
    uiModel: VehicleDiscountUiModel,
    onChangeConsumption: (consumptionText: String) -> Unit,
    onChangeTank: (tankText: String) -> Unit,
    onAddDiscount: (discount: DiscountDomainModel) -> Unit,
    onRemoveDiscount: (discount: DiscountDomainModel) -> Unit,
) {
    val scrollState = rememberLazyListState()
    
    Column(
        modifier = Modifier
            .fillMaxSize()
            .padding(Dimension.Padding.Normal),
    ) {
        LazyColumn(
            modifier = Modifier.fillMaxSize(),
            state = scrollState,
        ) {
            item {
                Row(
                    horizontalArrangement = Arrangement.spacedBy(Dimension.Padding.Normal)
                ) {
                    Icon(
                        modifier = Modifier
                            .size(24.dp)
                            .offset(y = Dimension.Padding.UltraVeryLittle),
                        painter = painterResource(id = R.drawable.ic_info),
                        contentDescription = null,
                    )
                    Text(
                        text = stringResource(id = R.string.vehicle_discount_info),
                        style = CustomTextStyle.Body,
                    )
                }
            }
            item {
                Text(
                    modifier = Modifier.padding(top = Dimension.Padding.VeryBig),
                    text = stringResource(id = R.string.vehicle_discount_consumption_label),
                    style = CustomTextStyle.TitleMedium,
                )
                CustomTextFieldComposable(
                    modifier = Modifier.padding(top = Dimension.Padding.LittleNormal),
                    value = uiModel.consumption.toString().replace('.', ','),
                    isValid = { it.isEmpty() || it.last() == ',' || it.replace(',', '.').toDoubleOrNull() != null },
                    suffixText = stringResource(id = R.string.vehicle_discount_consumption_hint),
                    onValueChange = onChangeConsumption,
                    keyboardOptions = KeyboardOptions.Default.copy(keyboardType = KeyboardType.Number),
                )
            }
            item {
                Text(
                    modifier = Modifier.padding(top = Dimension.Padding.VeryBig),
                    text = stringResource(id = R.string.vehicle_discount_tank_label),
                    style = CustomTextStyle.TitleMedium,
                )
                CustomTextFieldComposable(
                    modifier = Modifier.padding(top = Dimension.Padding.LittleNormal),
                    value = uiModel.tank.toString().replace('.', ','),
                    isValid = { it.toIntOrNull() != null },
                    suffixText = stringResource(id = R.string.vehicle_discount_tank_hint),
                    onValueChange = onChangeTank,
                    keyboardOptions = KeyboardOptions.Default.copy(keyboardType = KeyboardType.Number),
                )
            }
            item {
                var menuDiscountsExpanded by remember { mutableStateOf(false) }
                Text(
                    modifier = Modifier.padding(top = Dimension.Padding.VeryBig),
                    text = stringResource(id = R.string.vehicle_discount_discounts_label),
                    style = CustomTextStyle.TitleMedium,
                )
                Box(
                    modifier = Modifier.padding(bottom = Dimension.Padding.Big),
                ) {
                    Button(
                        modifier = Modifier.padding(top = Dimension.Padding.LittleNormal),
                        onClick = { menuDiscountsExpanded = true },
                        shape = RoundedCornerShape(Dimension.Border.Radius),
                        border = BorderStroke(
                            Dimension.Border.Stroke,
                            CustomColor.Black,
                        ),
                        colors = ButtonDefaults.buttonColors(
                            containerColor = CustomColor.White,
                            contentColor = CustomColor.Black,
                        ),
                        contentPadding = PaddingValues(Dimension.Padding.Normal)
                    ) {
                        Row(
                            horizontalArrangement = Arrangement.spacedBy(Dimension.Padding.Normal)
                        ) {
                            Text(
                                modifier = Modifier,
                                text = stringResource(id = R.string.general_add),
                                style = CustomTextStyle.Body,
                            )
                            Icon(
                                modifier = Modifier.size(18.dp),
                                painter = painterResource(id = R.drawable.ic_chevron_arrow_down),
                                contentDescription = null,
                            )
                        }
                    }
                    DropdownMenu(
                        modifier = Modifier.background(CustomColor.White),
                        expanded = menuDiscountsExpanded,
                        onDismissRequest = { menuDiscountsExpanded = false }
                    ) {
                        uiModel.discounts.forEach {
                            DropdownMenuItem(
                                modifier = Modifier.background(CustomColor.White),
                                text = {
                                    Text(
                                        text = it.name,
                                        style = CustomTextStyle.BodyBold
                                    )
                                },
                                onClick = {
                                    onAddDiscount(it)
                                    menuDiscountsExpanded = false
                                }
                            )
                        }
                    }
                }
            }
            items(
                count = uiModel.discountsEnable.size,
                key = {
                    "${uiModel.discountsEnable[it].id}"
                },
                itemContent = {
                    ItemDiscountComposable(
                        modifier = Modifier.padding(bottom = Dimension.Padding.Little),
                        item = uiModel.discountsEnable[it],
                        onRemoveDiscount = onRemoveDiscount,
                    )
                }
            )
        }
    }
}

@Composable
fun ItemDiscountComposable(
    item: DiscountDomainModel,
    onRemoveDiscount: (item: DiscountDomainModel) -> Unit,
    modifier: Modifier = Modifier,
) {
    Row(
        modifier = modifier.padding(bottom = Dimension.Padding.Normal),
        verticalAlignment = Alignment.CenterVertically,
        horizontalArrangement = Arrangement.spacedBy(Dimension.Padding.Normal),
    ) {
        IconButton(
            modifier = Modifier.size(24.dp),
            onClick = { onRemoveDiscount(item) },
        ) {
            Icon(
                modifier = Modifier.size(18.dp),
                painter = painterResource(id = R.drawable.ic_trash),
                contentDescription = null
            )
        }
        Text(
            modifier = Modifier,
            text = item.name,
            style = CustomTextStyle.BodyBold,
        )
        Text(
            modifier = Modifier,
            text = stringResource(
                id = R.string.vehicle_discount_discounts_type_percentage,
                item.value.toString(),
            ),
            style = CustomTextStyle.Body,
        )
    }
}

@Preview
@Composable
fun VehicleDiscountSuccessComposablePreview() {
    val discounts = ImmutableList.of(
        DiscountDomainModel(
            id = 5147,
            brand = BrandDomainModel(id = 3575, name = "Santa"),
            name = "Lourdes",
            description = "Kayle",
            recipient = "Montrell",
            value = 5382,
            type = DiscountTypeDomainEnum.CENTS,
            isActive = false
        ),
        DiscountDomainModel(
            id = 5732,
            brand = BrandDomainModel(id = 947, name = "Marinda"),
            name = "Max",
            description = "Krystyna",
            recipient = "Caley",
            value = 1618,
            type = DiscountTypeDomainEnum.PERCENTAGE,
            isActive = true
        )
    )
    VehicleDiscountSuccessComposable(
        uiModel = VehicleDiscountUiModel(
            consumption = 5.7,
            tank = 50,
            discounts = discounts,
            discountsEnable = discounts.filter { it.isActive }.toImmutableList(),
        ),
        onChangeConsumption = {},
        onChangeTank = {},
        onAddDiscount = {},
        onRemoveDiscount = {},
    )
}