package com.jrl.fuelprice.presentation.vehiclediscount

import com.google.common.collect.ImmutableList
import com.jrl.fuelprice.domain.discounts.models.DiscountDomainModel
import com.jrl.fuelprice.domain.discounts.usecases.GetDiscountsUseCase
import com.jrl.fuelprice.domain.discounts.usecases.UpdateDiscountUseCase
import com.jrl.fuelprice.domain.vehicledetails.usecases.GetVehicleDetailsUseCase
import com.jrl.fuelprice.domain.vehicledetails.usecases.SaveConsumptionUseCase
import com.jrl.fuelprice.domain.vehicledetails.usecases.SaveTankUseCase
import com.jrl.fuelprice.presentation.commons.extensions.toImmutableList
import com.jrl.fuelprice.presentation.commons.viewmodel.MviViewModel
import com.jrl.fuelprice.presentation.commons.viewmodel.UiState
import com.jrl.fuelprice.presentation.vehiclediscount.VehicleDiscountUiAction.Load
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject

@HiltViewModel
class VehicleDiscountViewModel @Inject constructor(
    private val getVehicleDetailsUseCase: GetVehicleDetailsUseCase,
    private val saveConsumptionUseCase: SaveConsumptionUseCase,
    private val saveTankUseCase: SaveTankUseCase,
    private val getDiscountsUseCase: GetDiscountsUseCase,
    private val updateDiscountUseCase: UpdateDiscountUseCase,
) : MviViewModel<UiState<VehicleDiscountUiModel>, VehicleDiscountUiAction, VehicleDiscountUiSingleEvent>() {
    
    private var consumption: Double = 0.0
    private var tank: Int = 0
    private var discounts: ImmutableList<DiscountDomainModel> = ImmutableList.of()
    private val discountsActives: ImmutableList<DiscountDomainModel>
        get() = discounts.filter { it.isActive }.toImmutableList()
    
    override fun handleAction(action: VehicleDiscountUiAction) {
        when (action) {
            Load -> loadInitialData()
            is VehicleDiscountUiAction.OnChangeConsumption -> onChangeConsumption(action.consumptionText)
            is VehicleDiscountUiAction.OnChangeTank -> onChangeTank(action.tankText)
            is VehicleDiscountUiAction.OnAddDiscount -> updateDiscount(action.discount, true)
            is VehicleDiscountUiAction.OnRemoveDiscount -> updateDiscount(action.discount, false)
        }
    }
    
    private fun loadInitialData() {
        launchIO {
            getVehicleDetailsUseCase.invoke().onSuccess {
                consumption = it.consumption
                tank = it.tank
                submitUiState()
            }
        }
        launchIO {
            getDiscountsUseCase.invoke().onSuccess {
                discounts = it.toImmutableList()
                submitUiState()
            }
        }
    }
    
    private fun onChangeConsumption(consumptionText: String) {
        launchIO {
            try {
                consumptionText.replace(",", ".").toDouble().let {
                    consumption = it
                    saveConsumptionUseCase.invoke(it)
                }
            } catch (e: NumberFormatException) {
                // Show error
            }
        }
    }
    
    private fun onChangeTank(tankText: String) {
        launchIO {
            try {
                tankText.toInt().let {
                    tank = it
                    saveTankUseCase.invoke(it)
                }
            } catch (e: NumberFormatException) {
                // Show error
            }
        }
    }
    
    private fun updateDiscount(discount: DiscountDomainModel, isActive: Boolean) {
        discounts = discounts.map {
            if (it.id == discount.id) discount.copy(isActive = isActive) else it
        }.toImmutableList()
        submitUiState()
        launchIO {
            updateDiscountUseCase.invoke(
                discountId = discount.id,
                isActive = isActive,
            )
        }
    }
    
    private fun submitUiState() {
        submitState(
            UiState.Success(
                VehicleDiscountUiModel(
                    consumption = consumption,
                    tank = tank,
                    discounts = discounts,
                    discountsEnable = discountsActives,
                )
            )
        )
    }
}
