package com.jrl.fuelprice.domain.alerts.usecases

import app.cash.turbine.test
import com.jrl.fuelprice.domain.alerts.AlertRepository
import com.jrl.fuelprice.domain.alerts.models.AlertGasStationDomainModel
import com.jrl.fuelprice.domain.alerts.models.AlertGasStationDomainModelTest
import com.jrl.fuelprice.domain.gasstations.models.FuelTypeDomainEnum
import com.jrl.fuelprice.domain.gasstations.models.LocationDomainModel
import com.jrl.fuelprice.domain.messaging.MessagingRepository
import com.jrl.fuelprice.shared.AbsBaseTest
import io.kotest.matchers.shouldBe
import io.mockk.coEvery
import io.mockk.mockk
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.test.runTest
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.params.ParameterizedTest
import org.junit.jupiter.params.provider.Arguments
import org.junit.jupiter.params.provider.MethodSource

class GetAlertsUseCaseTest : AbsBaseTest() {
    
    private lateinit var useCase: GetAlertsUseCase
    private val mockAlertRepository: AlertRepository = mockk()
    private val mockMessagingRepository: MessagingRepository = mockk()
    
    companion object {
        @JvmStatic
        fun getArguments() = listOf<Arguments>(
            Arguments.of(
                FuelTypeDomainEnum.DIESEL_A,                    // fuel type
                null,                                           // location
                "",                                             // user id
                0,                                              // alerts size
                emptyList<List<AlertGasStationDomainModel>>(),  // alerts expected
            ),
            Arguments.of(
                FuelTypeDomainEnum.DIESEL_A,
                LocationDomainModel(40.509, -3.644),
                "fake-user-id",
                2,
                listOf(
                    AlertGasStationDomainModelTest.fake(),
                    AlertGasStationDomainModelTest.fake(),
                ),
            ),
            Arguments.of(
                FuelTypeDomainEnum.DIESEL_PREMIUM,
                LocationDomainModel(40.509, -3.644),
                null,
                0,
                emptyList<List<AlertGasStationDomainModel>>(),
            ),
            Arguments.of(
                FuelTypeDomainEnum.GASOLINE_95_E5,
                LocationDomainModel(40.509, -3.644),
                "fake-user-id",
                1,
                listOf(
                    AlertGasStationDomainModelTest.fake(),
                ),
            ),
        )
    }
    
    @BeforeEach
    override fun setUp() {
        super.setUp()
        useCase = GetAlertsUseCase(
            alertRepository = mockAlertRepository,
            messagingRepository = mockMessagingRepository,
        )
    }
    
    @ParameterizedTest(name = "GIVEN fuel {0} and location {1} WHEN get alerts THEN return {3} alerts")
    @MethodSource("getArguments")
    fun `USE CASE get alerts`(
        fuelType: FuelTypeDomainEnum,
        location: LocationDomainModel?,
        userId: String?,
        alertsSize: Int,
        alertsExpected: List<AlertGasStationDomainModel>,
    ) = runTest {
        // 🔵 Given //
        coEvery {
            mockMessagingRepository.getToken()
        } returns userId
        if (userId != null) {
            coEvery {
                mockAlertRepository.getAlerts(
                    userId = userId,
                    fuelType = fuelType,
                    location = location,
                )
            } returns flow { emit(alertsExpected) }
        }
        
        // 🟡 When //
        val flow = useCase.invoke(
            fuelType = fuelType,
            location = location,
        )
        
        // 🟢 Then //
        flow.test {
            awaitItem().shouldBe(alertsExpected)
            awaitComplete()
        }
    }
}